import unittest

from .clipBoard import *
from . import *


class TestclipBoard(unittest.TestCase):

    def test_1(self):
        pass
        置剪辑板文本("易函数")

        self.assertEqual(取剪辑板文本(), "易函数")
        清除剪辑板()
        self.assertEqual(取剪辑板文本(), "")


    def test_2(self):
        pass
        # data = 取鼠标位置()
        # print(data.x)
        # print(data.y)



import unittest

from .processPoolUtil import *
from .__init__ import *


class TestProcessPoolUtil(unittest.TestCase):

    def test_1(self):
        pass
        def 任务函数(参数):
            print("任务函数",参数)
            return 参数

        任务池 = 进程池(4)
        result = []
        for url in range(10):
            task = 任务池.投递任务(任务函数, (url))
            result.append(task)

        延时(2)

        for res in result:
            try:
                data = 任务池.取返回值(res)
                print(data)
            except:
                print("出错了")

        任务池.等待()



    def test_2(self):
        def 任务函数(参数):
            print("任务函数",参数)
            return 参数

        # x = 进程(任务函数)
        # x.启动()
        # x.等待进程()




import wx
from .wxControl import *
from .Sound import *


class 窗口(wx.Frame, 公用方法):

    def 关闭(self):
        self.Close(True)

    @组件_异常检测
    def 置图标(self, 图标路径):
        icon = wx.Icon(图标路径)
        self.SetIcon(icon)

    def 居中(self):
        '将窗口调整到屏幕中间'
        self.Centre()

    @组件_异常检测
    def 创建状态栏(self, 项目, 抓取器=True):  # 只能创建一个
        "项目格式为list，每个成员为tuple 格式(项目名,宽度)，如[('项目1',-1),('项目2',-1)],负数表示按比例分配"
        try:
            if self.状态栏:
                return False
        except:
            pass

        样式 = wx.STB_SHOW_TIPS | wx.STB_ELLIPSIZE_END | wx.FULL_REPAINT_ON_RESIZE
        样式 += wx.STB_SIZEGRIP if 抓取器 else 0
        self.状态栏 = self.CreateStatusBar(style=样式)
        self.状态栏.SetFieldsCount(len(项目))
        self.状态栏.SetStatusWidths([x[1] for x in 项目])
        for x in range(len(项目)):
            self.状态栏.SetStatusText(项目[x][0], x)

    @组件_异常检测
    def 置状态栏项目宽度(self, 宽度列表):
        '需要传入一个整数列表设置全部项目宽度'
        self.状态栏.SetStatusWidths(宽度列表)

    @组件_异常检测
    def 置状态栏项目文本(self, 索引, 文本):
        self.状态栏.SetStatusText(文本, 索引)

    @组件_异常检测
    def 取状态栏项目数(self):
        return self.状态栏.GetFieldsCount()

    @组件_异常检测
    def 取状态栏项目文本(self, 索引):
        return self.状态栏.GetStatusText(索引)

    @组件_异常检测
    def 取状态栏项目宽度(self, 索引):
        return self.状态栏.GetStatusWidth(索引)

    @组件_异常检测
    def 取状态栏项目样式(self, 索引):
        return self.状态栏.GetStatusStyle(索引)

    @组件_异常检测
    def 置状态栏最小宽度(self, 宽度):
        return self.状态栏.SetMinHeight(宽度)

    @组件_异常检测
    def 置状态栏项目数(self, 项目数, 宽度列表=[]):
        宽度列表 = 宽度列表 if 宽度列表 else [-1 for x in range(项目数)]
        return self.状态栏.SetFieldsCount(项目数, 宽度列表)

    音乐播放次数 = "仅播放一次"
    sound = None

    @property
    def 播放次数(self):
        return self.音乐播放次数

    @播放次数.setter
    def 播放次数(self, value):
        self.音乐播放次数 = value
        if self.sound:
            self.背景音乐 = self.背景音乐路径

    @property
    def 背景音乐(self):
        return self.背景音乐路径

    @背景音乐.setter
    def 背景音乐(self, value):
        self.背景音乐路径 = value
        if self.背景音乐路径 == "":
            return

        if not self.sound:
            self.sound = 播放器(self.背景音乐路径)
        if self.sound.IsOk():
            if self.音乐播放次数 == "循环播放":
                flag = wx.adv.SOUND_ASYNC | wx.adv.SOUND_LOOP
                self.sound.Play(flag)
            elif self.音乐播放次数 == "仅播放一次":
                flag = wx.adv.SOUND_ASYNC
                self.sound.Play(flag)
            elif self.音乐播放次数 == "不播放":
                pass
        else:
            print("音乐文件加载失败 %s" % self.背景音乐路径)

    _窗口位置 = "居中"

    @property
    def 位置(self):
        return self._窗口位置

    @播放次数.setter
    def 位置(self, value):
        self._窗口位置 = value
        if self._窗口位置 == "通常":
            self.Centre()
        elif self._窗口位置 == "居中":
            self.Centre()
        elif self._窗口位置 == "最小化":
            self.Iconize(True)
        elif self._窗口位置 == "最大化":
            self.Maximize()
        elif self._窗口位置 == "全屏":
            self.ShowFullScreen(True)

    _图标 = ""

    @property
    def 图标(self):
        return self._图标

    @播放次数.setter
    def 图标(self, value):
        self._图标 = value
        if value == "":
            return
        self.置图标(value)

    _总在最前 = False

    @property
    def 总在最前(self):
        return self._总在最前

    @播放次数.setter
    def 总在最前(self, value=True):
        self._总在最前 = value
        if value:
            style = self.GetWindowStyle()
            self.SetWindowStyle(style | wx.STAY_ON_TOP)
        else:
            self.ToggleWindowStyle(wx.STAY_ON_TOP)

    def 置菜单栏(self, 菜单栏):
        self.SetMenuBar(菜单栏)



"""

.. Hint::
    日期时间操作 符合中国人思维逻辑的日期时间操作实用函数


.. literalinclude:: ../../../pyefun/timeBase_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

import time
import pendulum
import copy
from .stringBase import *


def 时间_设置语言包(名称="zh"):
    pendulum.set_locale(名称)


# 文档
# https://pendulum.eustace.io/docs/#instantiation
class 日期时间:
    t = pendulum.DateTime

    def __init__(self, str="now"):
        pass

        if (type(str) == int):
            self.t = pendulum.from_timestamp(str, tz='Asia/Shanghai')
        elif (str != ""):
            self.t = pendulum.parse(str)

    def 到文本(self, format="YYYY-MM-DD HH:mm:ss"):
        return self.t.format(format)

    def 取时间戳(self):
        return self.t.int_timestamp

    def 取日期(self):
        return self.t.to_date_string()

    def 取年(self):
        return self.t.year

    def 取月(self):
        return self.t.month

    def 取日(self):
        return self.t.day

    def 取小时(self):
        return self.t.hour

    def 取分钟(self):
        return self.t.minute

    def 取秒(self):
        return self.t.second

    def 取微妙(self):
        return self.t.microsecond

    def 取每周的第几天(self):
        return self.t.day_of_week

    def 取每年的第几天(self):
        return self.t.day_of_year

    def 取每月的第几周(self):
        return self.t.week_of_month

    def 取每年的第几周(self):
        return self.t.week_of_year

    def 取月天数(self):
        return self.t.days_in_month

    def 设置年月日(self, year=None, month=None, day=None):
        self.t = self.t.on(year=int(year), month=int(month), day=int(day))
        return self

    def 设置时分秒(self, hour=0, minute=0, second=0):
        self.t = self.t.at(hour, minute, second)
        return self

    def 增减日期时间(self, years=0, months=0, weeks=0, days=0, hours=0, minutes=0, seconds=0):
        self.t = self.t.add(years=years,
                            months=months,
                            weeks=weeks,
                            days=days,
                            hours=hours,
                            minutes=minutes,
                            seconds=seconds,
                            )
        return self

    def 取时间间隔(self, datetime):

        d = self.t.diff(datetime, abs=False)
        return {
            "years": d.in_years(),
            "months": d.in_months(),
            "days": d.in_days(),
            "hours": d.in_hours(),
            "minutes": d.in_minutes(),
            "seconds": d.in_seconds(),
            "weeks": d.in_weeks(),
        }

    def 取友好时间(self):
        d = self.t.diff_for_humans()
        return d

    def datetime(self):
        return self.t

    def copy(self):
        return copy.deepcopy(self)

    def __str__(self):
        return self.到文本()


def 取现行时间戳():
    return round(time.time())


def 日期到时间戳(str="now"):
    return 日期时间(str).取时间戳()


def 取现行时间():
    t = 日期时间()
    return t


def now():
    t = 日期时间()
    return t


def 创建日期时间(timestr):
    t = 日期时间(timestr)
    return t


def 时间迭代(start, end):
    """
    为支持单位range()有：years，months，weeks， days，hours，minutes和seconds
    for i in list.range('months'):
        print(i.to_datetime_string())
    """
    return pendulum.period(pendulum.parse(start), pendulum.parse(end))


def date(format, timestr="now"):
    t = 创建日期时间(timestr)
    format = 子文本替换(format, "Y", str(t.取年()))
    format = 子文本替换(format, "m", str(t.取月()))
    format = 子文本替换(format, "d", str(t.取日()))
    format = 子文本替换(format, "H", str(t.取小时()))
    format = 子文本替换(format, "i", str(t.取分钟()))
    format = 子文本替换(format, "s", str(t.取秒()))
    return format



import asyncio
import time
from asyncPool import *


async def 工作任务(i):
    await asyncio.sleep(1)
    return i


async def thread_example3(i):
    time.sleep(1)
    await asyncio.sleep(0.1)
    return i


def 工作任务4(i):
    time.sleep(0.01)
    return i


def 接受结果(future):
    result = future.result()
    print('返回值: ', result)


def demo():
    # 任务组， 最大协程数
    pool = 协程池(协程数量=10, 线程池数量=10)

    # 插入任务任务
    for i in range(1000):
        # print(i)
        # 非阻塞协程
        pool.投递任务(工作任务, i, 回调函数=接受结果)
        # 阻塞协程
        # pool.投递任务2(工作任务4, i, 回调函数=接受结果)

    print("等待子线程结束1...")

    # 停止子线程
    pool.释放线程()

    print("等待子线程结束2...")

    # 等待
    pool.等待()

    print("等待子线程结束3...")

    #
    # # 进度条使用
    # while True:
    #     value = pool.task_progress(99)
    #     print("value: ", value)
    #
    #     # 获取线程数
    #     print("running: ", pool.running)
    #
    #     if value == 100:
    #         break
    #
    #     # 获取任务
    #     # print(pool.get_task())
    #     time.sleep(0.5)
    #
    # print("等待子线程结束4...")


if __name__ == '__main__':
    start_time = time.time()
    demo()
    end_time = time.time()
    print("run time: ", end_time - start_time)



"""
.. Hint:: 公用模块
    异常处理等等


.. code-block:: python
   :linenos:

   @异常处理返回类型逻辑型
   def 如果函数发生错误即可提示兵忽略:
        print("没问题")
        a = 0/0 # 这个一定报错


"""
import traceback, datetime

异常显示信息 = True


def 设置_异常处理_显示信息(显示信息=2):
    """

    :param 显示信息: 0 不显示 1显示简单的 2显示详细的
    :return:
    """
    global 异常显示信息
    异常显示信息 = 显示信息


def 异常处理返回类型逻辑型(function):
    """
    使用方法 在def上面加上 @异常处理返回类型逻辑型

    例如

.. code-block:: python
   :linenos:

    @异常处理返回类型逻辑型
    def 如果函数发生错误即可提示兵忽略:
        print("没问题")
        a = 0/0 # 这个一定报错

    """

    def box(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except:
            if 异常显示信息 == 2:
                print("函数异常 %s 时间 %s \r\n%s" % (
                    function.__name__,
                    str(datetime.datetime.now()),
                    traceback.format_exc()
                ))
            if 异常显示信息 == 1:
                print("函数异常 %s 时间 %s" % (
                    function.__name__,
                    str(datetime.datetime.now()),
                ))

            return False

    return box



"""
菜单栏

[
  {
    "名称": "文件",
    "子菜单": [
      {
        "名称": "新建",
        "图标": "C:/pyefun/wxview/images/工具条/新建.png",
        "绑定事件": "菜单_新建_被点击"
      },
      {
        "名称": "打开",
        "图标": "C:/pyefun/wxview/images/工具条/打开.png"
      },
      {
        "名称": "关闭",
        "图标": "C:/pyefun/wxview/images/工具条/关闭.png"
      },
      {
        "名称": "退出\tCtrl+S",
        "图标": "C:/pyefun/wxview/images/工具条/保存.png"
      },
      {
        "名称": "另存为",
        "图标": "C:/pyefun/wxview/images/工具条/另存为.png",
        "隐藏": true
      },
      {
        "名称": "自动保存2",
        "选中": false,
        "类型": "单选框"
      },
      {
        "名称": "自动保存1",
        "选中": true,
        "类型": "单选框"
      },
      {
        "名称": "自动保存",
        "选中": true,
        "类型": "选择框"
      },
      {
        "名称": "退出\tCtrl+Q",
        "id": "wx.ID_EDIT",
        "图标": "C:/pyefun/wxview/images/工具条/关闭.png",
        "绑定事件": "菜单_退出_被点击",
        "帮助文本": "退出系统"
      }
    ]
  },
  {
    "名称": "其他",
    "子菜单": [
      {
        "名称": "其他",
        "子菜单": [
          {
            "名称": "其他11"
          },
          {
            "名称": "其他22"
          },
          {
            "名称": "其他33"
          },
          {
            "名称": "其他44"
          }
        ]
      },
      {
        "名称": "其他2"
      },
      {
        "名称": "其他3"
      },
      {
        "名称": "其他4"
      }
    ]
  }
]

    菜单数据 = 读入文本("C:\pyefun\wxview\menu.json")
    菜单栏 = wx.菜单栏()
    菜单栏.从菜单数据中创建(菜单数据, self)
    self.置菜单栏(菜单栏)


"""
import wx
from .wxControl import *
import pyefun as efun
import json

ID_START = 1000


def _getMenuId():
    global ID_START
    ID_START += 1
    return ID_START


class 菜单栏(wx.MenuBar, 公用方法):
    pass

    def 从菜单数据中创建(self, 创建菜单的json数据, winframe=wx.Frame):
        menuJson = json.loads(创建菜单的json数据)
        menuBar = self
        for 第一层的值 in menuJson:
            print(第一层的值.get("名称"))
            print(第一层的值.get("子菜单"))
            menu = menuBar.创建多级菜单(第一层的值.get("子菜单"), winframe)
            menuBar.Append(menu, 第一层的值.get("名称"))

    def 创建多级菜单(self, 创建菜单的json数据, winframe=wx.Frame):
        menu = wx.Menu()
        for 菜单值 in 创建菜单的json数据:
            k = 菜单值.get("名称")
            v = 菜单值.get("子菜单")
            图标 = 菜单值.get("图标")
            禁用 = 菜单值.get("禁用")
            隐藏 = 菜单值.get("隐藏")
            id = 菜单值.get("id")
            绑定事件 = 菜单值.get("绑定事件")
            选中 = 菜单值.get("选中")
            类型 = 菜单值.get("类型")
            帮助文本 = 菜单值.get("帮助文本")
            print(菜单值.get("名称"))
            print(菜单值.get("子菜单"))
            # 分隔符
            if k == '-':
                menu.AppendSeparator()
                continue
            # 子菜单
            if type(v) is list:
                submenu = self.创建多级菜单(v, winframe)
                menu.AppendSubMenu(submenu, k)
                return menu

            # 创建一个菜单的过程
            if id is None:
                id = _getMenuId()
            else:
                try:
                    if (type(id) == str):
                        id = eval(id)
                except:
                    print("建立菜单[{}]失败,无法获取菜单id[{}]".format(k, id))
                    id = _getMenuId()

            kind = wx.ITEM_NORMAL
            if 类型 is not None:
                if 类型 == "单选框":
                    kind = wx.ITEM_RADIO
                if 类型 == "选择框":
                    kind = wx.ITEM_CHECK

            if 帮助文本 is None:
                帮助文本 = ""

            item = wx.MenuItem(menu, id, k, 帮助文本, kind)

            if 图标 is not None and 图标 != "":
                图标 = efun.子文本替换(图标, "./", efun.取运行目录() + "/")
                图标 = efun.路径优化(图标)
                if efun.文件是否存在(图标):
                    item.SetBitmap(wx.Image(图标).Scale(16, 16).ConvertToBitmap())
            if 禁用 is True:
                item.Enable(False)
            if 隐藏 is True:
                continue
            if 绑定事件 is not None:
                try:
                    fc = "winframe.{}".format(绑定事件)
                    winframe.Bind(wx.EVT_MENU, eval(fc), item)
                except:
                    print("未定义菜单被点击事件[{}]函数  菜单名[{}]".format(fc, k))
            menu.Append(item)
            if 选中 is not None:
                item.Check(选中)
        return menu



"""

.. Hint::
    实用函数

.. literalinclude:: ../../../pyefun/commonlyUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""
import random

import ubelt as ub
from collections import OrderedDict
import operator
from .stringUtil import *


def 取sha1(data, 哈希算法='sha1'):
    return ub.hash_data(data, hasher=哈希算法)

def 取哈希(data, 哈希算法='sha1'):
    return ub.hash_data(data, hasher=哈希算法)


def 取文件哈希(文件路径, 哈希算法='sha1'):
    return ub.hash_file(文件路径, hasher=哈希算法, base='hex')


def 运行命令(str,
         运行目录=None,
         环境变量=None,
         显示信息=0,
         后台运行=False,
         shell=False
         ):
    info = ub.cmd(str,
                  cwd=运行目录,
                  env=环境变量,
                  verbose=显示信息,
                  detach=后台运行,
                  shell=shell,
                  )
    return (info['out'])


def 取缓存目录(name):
    """
    # The resource root directory is
    # ~/AppData/Roaming on Windows,
    # ~/.config on Linux and ~/Library/Application Support on Mac.
    # The cache root directory is ~/AppData/Local on Windows,
    # ~/.config on Linux and ~/Library/Caches on Mac.
    """
    return ub.shrinkuser(ub.ensure_app_cache_dir(name))


def 下载文件(url, 保存文件路径=None):
    fpath = ub.download(url, fpath=保存文件路径, verbose=0)
    return ub.shrinkuser(fpath)


def 下载文件缓存(url, 保存文件路径=None):
    fpath = ub.grabdata(url, fpath=保存文件路径, verbose=0)
    return ub.shrinkuser(fpath)


def 字典_取最小值(indexable, key=None):
    """
    # assert argmin({'a': 3, 'b': 2, 'c': 100}) == 'b'
    # assert argmin(['a', 'c', 'b', 'z', 'f']) == 0
    # assert argmin([[0, 1], [2, 3, 4], [5]], key=len) == 2
    # assert argmin({'a': 3, 'b': 2, 3: 100, 4: 4}) == 'b'
    # assert argmin(iter(['a', 'c', 'A', 'z', 'f'])) == 2
    """
    return ub.argmin(indexable, key)


def 字典_分组(数组, 类别数组):
    groups = ub.group_items(数组, 类别数组)
    return groups


def 字典_统计(数组, weights=None, ordered=False, labels=None):
    data = ub.dict_hist(数组, weights, ordered, labels)
    return data


def 字典_取子集(数组, key, default=ub.util_const.NoParam, cls=OrderedDict):
    data = ub.dict_subset(数组, key, default, cls)
    return data


def 字典_取值(字典, key, default=None):
    return list(ub.dict_take(字典, key, default))


def 字典_合并(*args):
    # 字典_取值({'a': 1, 'b': 1}, {'b': 2, 'c': 2})
    data = ub.dict_union(*args)
    return data


def 字典_差集(*args):
    # 字典_差集({'a': 1, 'b': 1}, {'a'}, {'c'})
    data = ub.dict_diff(*args)
    return data


def 字典_根据值重建(func, dict_):
    data = ub.map_vals(func, dict_)
    return data


def 字典_根据健重建(func, dict_):
    data = ub.map_keys(func, dict_)
    return data


def 字典_根据值排序(dict_, key=None, reverse=False):
    # dict_ = {'spam': 2.62, 'eggs': 1.20, 'jam': 2.92}
    # newdict = 字典_排序(dict_)
    # print(ub.repr2(newdict, nl=0))
    # {'eggs': 1.2, 'spam': 2.62, 'jam': 2.92}
    # newdict = 字典_排序(dict_, reverse=True)
    # print(ub.repr2(newdict, nl=0))
    # {'jam': 2.92, 'spam': 2.62, 'eggs': 1.2}
    # newdict = 字典_排序(dict_, key=lambda x: x % 1.6)
    # print(ub.repr2(newdict, nl=0))
    # {'spam': 2.62, 'eggs': 1.2, 'jam': 2.92}
    data = ub.sorted_vals(dict_, key, reverse)
    return data


def 字典_根据键排序(dict_, key=None, reverse=False):
    # import ubelt as ub
    # dict_ = {'spam': 2.62, 'eggs': 1.20, 'jam': 2.92}
    # newdict = sorted_keys(dict_)
    # print(ub.repr2(newdict, nl=0))
    # {'eggs': 1.2, 'jam': 2.92, 'spam': 2.62}
    # newdict = sorted_keys(dict_, reverse=True)
    # print(ub.repr2(newdict, nl=0))
    # {'spam': 2.62, 'jam': 2.92, 'eggs': 1.2}
    # newdict = sorted_keys(dict_, key=lambda x: sum(map(ord, x)))
    # print(ub.repr2(newdict, nl=0))
    # {'jam': 2.92, 'eggs': 1.2, 'spam': 2.62}
    data = ub.sorted_keys(dict_, key, reverse)
    return data


def 字典_交换健值(dict_, 唯一值=True):
    data = ub.invert_dict(dict_, unique_vals=唯一值)
    return data


def 字典_查找重复项(items, 至少出现=2, key=None):
    data = ub.util_dict.find_duplicates(items, 至少出现, key)
    return data


def 灵活字典():
    return ub.AutoDict()


def 灵活有序字典():
    return ub.AutoOrderedDict()


def 数组_合并为字典(items1, items2, cls=dict):
    return ub.dzip(items1, items2, cls)


def 导入包_从路径(路径):
    return ub.import_module_from_path(路径)


def 创建连接(文件路径, 目标路径, 覆盖=False, 显示信息=0):
    return ub.symlink(文件路径, 目标路径, 覆盖, 显示信息)


def 数组_查找重复项(items, 至少出现=2):
    data = ub.find_duplicates(items, k=至少出现)
    return data



def 命令行_获取参数(参数名, 默认值=ub.util_const.NoParam, argv=None):
    return ub.argval(参数名, 默认值, argv)


def 命令行_是否存在参数(参数名, argv=None):
    return ub.argflag(参数名, argv)


def 内存缓存(func):
    # 缓存函数结果
    return ub.memoize(func)


def 内存缓存方法(func):
    # 缓存方法函数结果
    return ub.memoize_method(func)


def 内存缓存属性(func):
    # 只会执行1次不可变更
    return ub.memoize_property(func)


def 路径_名字处理(路径, 末尾='', 前缀='', 扩展名=None, 名称=None, dpath=None,
            relative=None, 不包含点=False):
    return ub.augpath(路径, 末尾, 前缀, 扩展名, 名称, dpath, relative, 不包含点)


def 路径_取用户目录(用户名=None):
    # 返回某个用户主目录的路径。
    return ub.userhome(用户名)


def 路径_替换为用户路径(path, home='~'):
    # 返回某个用户主目录的路径。
    return ub.shrinkuser(path, home)


def 路径_展开路径(path):
    r"""

    :param path:  ~/foo
    :return: C:\Users\foo
    """

    return 路径_优化路径(ub.expandpath(path))


def 路径_优化路径(path):
    """
    把\\ // 乱七八糟的路径转化为规整的

    :param path:
    :return:
    """
    if 寻找文本(path, "\\") > -1:
        path = 子文本替换(path, "\\", "/")
    if 寻找文本(path, r"\\") > -1:
        path = 子文本替换(path, r"\\", "/")
    if 寻找文本(path, r"//") > -1:
        path = 子文本替换(path, r"//", "/")
    return ub.util_path.normpath(path)


def 路径_合并(*path):
    """
    a b c d 合并为 a/b/c/d 并且自动优化路径
    :param path:
    :return:
    """
    return 路径_优化路径(ub.util_path.join(*path))

def 路径_拼接(*path):
    return 路径_合并(*path)

def 目录_创建(路径, 权限=0o1777, 显示信息=None, 重建=False):
    return ub.ensuredir(路径, 权限, 显示信息, 重建)


class 临时目录(ub.TempDir):
    """
    用于创建和清理临时目录的上下文。

    #Example:
    #    with 临时目录() as self:
    #        dpath = self.dpath
    #        print(dpath)
    #        assert 文件是否存在(dpath)
    #    assert not 文件是否存在(dpath)

    #Example:
    #    self = 临时目录()
    #    dpath = self.ensure()
    #    assert exists(dpath)
    #    self.cleanup()
    #    assert not exists(dpath)
    """

    def 初始化(self):
        return self.ensure()

    def 清理(self):
        self.cleanup()

    def 取路径(self):
        return self.dpath


def 系统_取用户数据目录():
    return ub.platform_data_dir()


def 系统_取配置目录():
    return ub.platform_config_dir()


def 系统_缓存目录():
    return ub.platform_cache_dir()


def 系统_设置应用数据目录(appname, *args):
    return ub.ensure_app_data_dir(appname, *args)


def 系统_取应用配置目录(appname, *args):
    return ub.get_app_config_dir(appname, *args)


def 系统_设置应用配置目录(appname, *args):
    return ub.ensure_app_config_dir(appname, *args)


def 系统_取应用缓存目录(appname, *args):
    return ub.get_app_cache_dir(appname, *args)


def 系统_设置应用缓存目录(appname, *args):
    return ub.ensure_app_cache_dir(appname, *args)


def 查找可执行文件(名称, 匹配所有=False, 路径=None):
    """
    # 查找可执行文件('ls')
    # 查找可执行文件('ping')
    # assert 查找可执行文件('which') == 查找可执行文件(查找可执行文件('which'))
    # 查找可执行文件('which', 匹配所有=True)
    # 查找可执行文件('ping', 匹配所有=True)
    # 查找可执行文件('cmake', 匹配所有=True)
    # 查找可执行文件('nvcc', 匹配所有=True)
    # 查找可执行文件('noexist', 匹配所有=True)
    """
    return ub.find_exe(名称, 匹配所有, 路径)


def 查找文件或目录(名称, 路径=None, 精确=False):
    """
    # list(查找文件或目录('ping', exact=True))
    # list(查找文件或目录('bin'))
    # list(查找文件或目录('bin'))
    # list(查找文件或目录('*cc*'))
    # list(查找文件或目录('cmake*'))
    """
    return ub.find_path(名称, 路径, 精确)


def 文本_缩进(文本, 前缀='    '):
    return ub.util_str.indent(文本, 前缀)


def 文本_代码块(文本):
    return ub.util_str.codeblock(文本)


def 文本_段落(文本):
    return ub.util_str.paragraph(文本)


def 文本_水平合并(args, sep=''):
    """
    # import ubelt as ub
    # B = ub.repr2([[1, 2], [3, 457]], nl=1, cbr=True, trailsep=False)
    # C = ub.repr2([[5, 6], [7, 8]], nl=1, cbr=True, trailsep=False)
    # args = ['A = ', B, ' * ', C]
    # print(文本_水平合并(args))
    # A = [[1, 2],   * [[5, 6],
    #               [3, 457]]    [7, 8]]
    """
    return ub.util_str.hzcat(args, sep)


def 文本_转unicode(str):
    return ub.ensure_unicode(str)


class 控制台(ub.CaptureStdout):
    """
    控制台操作
    """

    def __init__(self, 获取内容=True, 是否启用=True):
        super().__init__(supress=获取内容, enabled=是否启用)

    def 停止(self):
        self.stop()

    def 开始(self):
        self.start()

    def 获取内容(self):
        return self.text.strip()


class 分块(ub.chunks):
    pass

    def __init__(self, items, 分块数量=None, 创建块数=None, 创建数量=None,
                 边界模式='none'):
        """
        # 边界模式（str）–确定输入的长度不能被块大小整除的最后一种情况，
        # 有效值为：{'none'，'cycle'，'replicate'}
"""
        super().__init__(
            items=items,
            chunksize=分块数量,
            nchunks=创建块数,
            total=创建数量,
            bordermode=边界模式,
        )


def 数组_索引取值(items, indices, default=ub.util_const.NoParam):
    return ub.util_list.take(items, indices, default)


def 数组_逻辑取值(items, flags):
    return ub.util_list.compress(items, flags)


def 数组_转平面(items):
    return ub.util_list.flatten(items)


def 数组_去重复(items, key=None):
    """
    # import ubelt as ub
    # import six
    # items = ['A', 'a', 'b', 'B', 'C', 'c', 'D', 'e', 'D', 'E']
    # unique_items = list(ub.unique(items, key=six.text_type.lower))
    # assert unique_items == ['A', 'b', 'C', 'D', 'e']
    # unique_items = list(ub.unique(items))
    # assert unique_items == ['A', 'a', 'b', 'B', 'C', 'c', 'D', 'e', 'E']
    """
    return ub.util_list.unique(items, key)


def 数组_取唯一值的索引(items, key=None):
    """
    # import ubelt as ub
    # import six
    # items = ['A', 'a', 'b', 'B', 'C', 'c', 'D', 'e', 'D', 'E']
    # unique_items = list(ub.unique(items, key=six.text_type.lower))
    # assert unique_items == ['A', 'b', 'C', 'D', 'e']
    # unique_items = list(ub.unique(items))
    # assert unique_items == ['A', 'a', 'b', 'B', 'C', 'c', 'D', 'e', 'E']
    """
    return ub.util_list.argunique(items, key)


def 数组_取唯一值的逻辑值(items, key=None):
    """
    # import ubelt as ub
    # import six
    # items = ['A', 'a', 'b', 'B', 'C', 'c', 'D', 'e', 'D', 'E']
    # unique_items = list(ub.unique(items, key=six.text_type.lower))
    # assert unique_items == ['A', 'b', 'C', 'D', 'e']
    # unique_items = list(ub.unique(items))
    # assert unique_items == ['A', 'a', 'b', 'B', 'C', 'c', 'D', 'e', 'E']
    """
    return ub.util_list.unique_flags(items, key)


def 数组_构建逻辑值列表(indices, maxval=None):
    """
    #    import ubelt as ub
    #indices = [0, 1, 4]
    #mask = ub.boolmask(indices, maxval=6)
    #assert mask == [True, True, False, False, True, False]
    #mask = ub.boolmask(indices)
    #assert mask == [True, True, False, False, True]
    """
    return ub.util_list.boolmask(indices, maxval)


def 数组_是否全部相同(iterable, eq=operator.eq):
    """
    #allsame([1, 1, 1, 1])
    # True
    #allsame([])
    # True
    #allsame([0, 1])
    # False
    #iterable = iter([0, 1, 1, 1])
    #next(iterable)
    #allsame(iterable)
    # True
    #allsame(range(10))
    # False
    #allsame(range(10), lambda a, b: True)
    # True
    """
    return ub.allsame(iterable, eq)


def 数组_排序索引(indexable, key=None, reverse=False):
    return ub.argsort(indexable, key, reverse)


def 数组_取最小值(indexable, key=None):
    """


    # assert argmin({'a': 3, 'b': 2, 'c': 100}) == 'b'
    # assert argmin(['a', 'c', 'b', 'z', 'f']) == 0
    # assert argmin([[0, 1], [2, 3, 4], [5]], key=len) == 2
    # assert argmin({'a': 3, 'b': 2, 3: 100, 4: 4}) == 'b'
    # assert argmin(iter(['a', 'c', 'A', 'z', 'f'])) == 2
    """
    return ub.argmin(indexable, key)


def 数组_弹出(iterable):
    return ub.peek(iterable)



import unittest

from .timeBase import *


class TestTime(unittest.TestCase):

    def test_1(self):
        pass
        t = 取现行时间()
        print(t)
        print(t.到文本("YYYY年MM月DD日 HH时mm分ss秒"))
        print(t.到文本())
        print("取年", t.取年())
        print("取月", t.取月())
        print("取日", t.取日())
        print("取小时", t.取小时())
        print("取分钟", t.取分钟())
        print("取秒", t.取秒())
        print("取微妙", t.取微妙())
        print("取每周的第几天", t.取每周的第几天())
        print("取每年的第几周", t.取每年的第几周())
        print("取每年的第几天", t.取每年的第几天())
        print("取每月的第几周", t.取每月的第几周())
        print("取日期", t.取日期())
        t.设置年月日(2020, 1, 1)
        print(t.到文本())
        t.设置时分秒(1, 1, 1)
        print(t.到文本())
        t = t.增减日期时间(
            years=1,
            months=1,
            days=1,
            hours=1,
            minutes=1,
            seconds=1,
        )
        print("增减日期时间", t.到文本())

        t1 = 取现行时间()
        t2 = t1.copy()
        print("取现行时间", t1.到文本())
        t2 = t2.增减日期时间(
            years=1,
            months=1,
            days=1,
            hours=1,
            minutes=1,
            seconds=1,
        ).datetime()
        print("t2", t2)
        print("t1", t1)
        print("取时间间隔", t1.取时间间隔(t2))

        t2 = 取现行时间()
        t2 = t2.增减日期时间(
            years=-1,
            months=1,
            days=1,
            hours=1,
            minutes=1,
            seconds=1,
        )
        print("取友好时间", t2.取友好时间())

        t = 取现行时间().取时间戳()
        print(t)

        t = 日期时间(取现行时间戳())
        print(t)
        print(t.取时间戳())

        t = 日期时间("2021-05-17 12:01:42").取时间戳()
        print(t)

        t = 日期到时间戳("2021-05-17 12:01:42")
        print(t)

    def test_2(self):
        list = 时间迭代("2021-01-17", "2021-05-27")
        for i in list.range('months'):
            print(i.to_datetime_string())

        # for i in list.range('days'):
        #     print(i.to_datetime_string())

    def test_3(self):
        data = date("Y-m-d H:i:s", now().取时间戳())
        print(data)




import wx
from .wxControl import *

class 颜色选择器(wx.ColourPickerCtrl, 公用方法):
    pass

    def 取当前颜色(self):
        return self.GetColour()

    @组件_异常检测
    def 置当前颜色(self,颜色):
        return self.SetColour(颜色)



import unittest

from .regexpUtil import *
import ubelt as ub


class TestRegexpUtil(unittest.TestCase):

    def test1(self):
        # 精易模块的实现方式
        pass
        str = r"""
        
aaa111bbb444ccc
aaa222bbb555ccc
aaa333bbb666ccc
        """
        zz = 正则表达式类(r"aaa(.*?)bbb(.*?)ccc", str)
        for i in range(zz.取匹配数量()):
            print(zz.取匹配文本(i))
            print(zz.取子匹配数量())
            print(zz.取子匹配文本(i, 0))
            print(zz.取子匹配文本(i, 1))
            print(zz.取子匹配文本(i, 2))
        print("================")

        str = r"""aaa111bbb444ccc
        aaa222bbb555ccc
        aaa333bbb666ccc
        """
        zz = 正则表达式类(r"\d+")
        data = zz.替换("==", str)
        print(data)

        正则结果 = 正则表达式类().创建(r"aaa(.*?)bbb(.*?)ccc").匹配(str).取结果()
        print(正则结果)

        正则结果 = 正则表达式类(r"aaa(.*?)bbb(.*?)ccc", str).取结果()
        print(正则结果)

        正则结果 = 正则表达式类().创建(r"aaa(.*?)bbb(.*?)ccc").替换("替换", str)
        print(正则结果)

        # 与上面不同的是..这里会搜索
        正则结果 = 正则表达式类(r"aaa(.*?)bbb(.*?)ccc", str).替换("替换")
        print(正则结果)

    def test_jingyi(self):
        # 与上面不同的是..这里会搜索
        str = r"""aaa111bbb444ccc
        aaa222bbb555ccc
        aaa333bbb666ccc
        """
        正则结果 = 正则表达式类().创建(r"aaa(.*?)bbb(.*?)ccc", str, 是否区分大小写=True, 是否匹配多行=True).取结果()
        print(正则结果)

        正则结果 = 正则表达式类().创建(r"\d+", str, 是否区分大小写=True, 是否匹配多行=True).替换("替换")
        print(正则结果)

        zz = 正则表达式类()
        zz.创建(r"\d+", str, 是否区分大小写=True, 是否匹配多行=True)
        for i in range(zz.取匹配数量()):
            print("取匹配文本", zz.取匹配文本(i))
            print("取子匹配数量", zz.取子匹配数量())
            for k in range(zz.取子匹配数量()):
                print("取子匹配文本 i= %s k= %s" % (i, k), zz.取子匹配文本(i, k))

    def test_my(self):
        str = r"""aaa111bbb444ccc
        aaa222bbb555ccc
        aaa333bbb666ccc
        AAA444BBB777CCC
        """
        正则结果 = 正则表达式(r"aaa(.*?)bbb(.*?)ccc").搜索(str)
        print(正则结果)

        正则结果 = 正则表达式(r"aaa(.*?)bbb(.*?)ccc").替换("替换", str)
        print(正则结果)

        正则结果 = 正则表达式(r"\d").替换("替换", str)
        print(正则结果)

        正则结果 = 正则表达式(r"AAA\d+BBB", re.M).替换("替换", str)
        print(正则结果)

        正则结果 = 正则表达式(r"AAA(\d+)BBB", re.M).搜索(str)
        print(正则结果)

        正则结果 = 正则表达式(r"AAA(\d+)BBB", 正则.多行模式 | 正则.忽略大小写).搜索(str)
        print(正则结果)

        # 只匹配第一项
        正则结果 = 正则表达式(r"AAA(\d+)BBB", 0).搜索(str)
        print(正则结果)

        正则结果 = 正则表达式(r"AAA(\d+)BBB", 正则.允许注释 | 正则.忽略大小写).搜索(str)
        print(正则结果)

        正则结果 = 正则表达式(r"""
        \d+ # 匹配数字
        """, 正则.多行模式 | 正则.忽略大小写 | 正则.允许注释).搜索(str)
        print(正则结果)


        print(正则表达式(r"\d+", 正则.多行模式 | 正则.忽略大小写).搜索("123456789"))
        print(正则表达式(r"\d+", 正则.多行模式 | 正则.忽略大小写).替换("替换", "123456798"))




"""

.. Hint::
    环境变量

    读入.env并且加载到系统环境变量中

.. literalinclude:: ..\..\..\example\python读取env文件.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-40

"""
from dotenv import dotenv_values, load_dotenv
import io
import os

def 环境变量_从文本中解析(env文件数据):
    fp = io.StringIO(env文件数据)
    config = dotenv_values(stream=fp)
    fp.close()
    return config

def 环境变量_从文本中加载至系统(env文件数据, 覆盖=True) -> bool:
    fp = io.StringIO(env文件数据)
    config = load_dotenv(stream=fp, override=覆盖)
    fp.close()
    return config

def 取系统所有环境变量():
    return os.environ

def 取环境变量(环境变量名称: str) -> str:
    """
        返回文本，它关连于一个操作系统环境变量。成功时返回所取得的值，失败则返回空文本。本命令为初级命令。
    """
    return os.environ.get(环境变量名称)

def 置环境变量(环境变量名称: str, 欲写入内容: str) -> bool:
    """
    修改或建立指定的操作系统环境变量。成功返回真，失败返回假。本命令为初级命令。
    """
    return os.environ.setdefault(环境变量名称, 欲写入内容)


def 环境变量_获取(环境变量名称: str) -> str:
    """
        返回文本，它关连于一个操作系统环境变量。成功时返回所取得的值，失败则返回空文本。本命令为初级命令。
    """
    return os.environ.get(环境变量名称)

def 环境变量_设置(环境变量名称: str, 欲写入内容: str) -> bool:
    """
    修改或建立指定的操作系统环境变量。成功返回真，失败返回假。本命令为初级命令。
    """
    return os.environ.setdefault(环境变量名称, 欲写入内容)



"""
工具条

[
  {
    "名称": "文件",
    "子菜单": [
      {
        "名称": "新建",
        "图标": "C:/pyefun/wxview/images/工具条/新建.png",
        "绑定事件": "菜单_新建_被点击"
      },
      {
        "名称": "打开",
        "图标": "C:/pyefun/wxview/images/工具条/打开.png"
      },
      {
        "名称": "关闭",
        "图标": "C:/pyefun/wxview/images/工具条/关闭.png"
      },
      {
        "名称": "退出\tCtrl+S",
        "图标": "C:/pyefun/wxview/images/工具条/保存.png"
      },
      {
        "名称": "另存为",
        "图标": "C:/pyefun/wxview/images/工具条/另存为.png",
        "隐藏": true
      },
      {
        "名称": "自动保存2",
        "选中": false,
        "类型": "单选框"
      },
      {
        "名称": "自动保存1",
        "选中": true,
        "类型": "单选框"
      },
      {
        "名称": "自动保存",
        "选中": true,
        "类型": "选择框"
      },
      {
        "名称": "退出\tCtrl+Q",
        "id": "wx.ID_EDIT",
        "图标": "C:/pyefun/wxview/images/工具条/关闭.png",
        "绑定事件": "菜单_退出_被点击",
        "帮助文本": "退出系统"
      }
    ]
  },
  {
    "名称": "其他",
    "子菜单": [
      {
        "名称": "其他",
        "子菜单": [
          {
            "名称": "其他11"
          },
          {
            "名称": "其他22"
          },
          {
            "名称": "其他33"
          },
          {
            "名称": "其他44"
          }
        ]
      },
      {
        "名称": "其他2"
      },
      {
        "名称": "其他3"
      },
      {
        "名称": "其他4"
      }
    ]
  }
]


"""

import wx
import json
import pyefun as efun
from .wxControl import *

ID_START = 2000


def _getMenuId():
    global ID_START
    ID_START += 1
    return ID_START


class 工具条(wx.ToolBar, 公用方法):
    pass

    def 设置图标大小(self, 宽度, 高度):
        self.SetToolBitmapSize((宽度, 高度))

    def 设置边距(self, 左边距, 上边距):
        self.SetMargins(x=左边距, y=上边距)

    def 设置间距(self, 间距=1):
        """如果工具栏是水平的，则用于垂直方向的间距，如果工具栏是垂直的，则用于水平方向的间距。"""
        self.SetToolPacking(间距)

    def 设置分割线大小(self, 间隔=5):
        self.SetToolSeparation(间隔)

    def 取工具对象(self, id):
        return self.FindById(id)

    def 取帮助文本(self, toolId):
        """传入对象或者传入工具按钮的id都可以"""
        if type(toolId) == int:
            return self.GetToolShortHelp(toolId)
        else:
            return self.GetToolShortHelp(toolId.GetId())

    def 取工具名称(self, item对象):
        return item对象.GetLabel()

    def 刷新显示(self):
        self.Realize()

    def 从工具条数据中创建(self, 工具条数据, 图标宽度=32, 图标高度=32):
        toolJson = json.loads(工具条数据)
        for 第一层的值 in toolJson:
            id = 第一层的值.get("id")
            名称 = 第一层的值.get("名称")
            图标 = 第一层的值.get("图标")
            帮助文本 = 第一层的值.get("帮助文本")
            if 名称 == "-":
                self.AddSeparator()
                continue

            if 帮助文本 is None:
                帮助文本 = 名称

            if id is None:
                id = _getMenuId()
            else:
                id = eval(id)
            if 图标 is not None:
                图标 = efun.子文本替换(图标, "./", efun.取运行目录() + "/")
                图标 = efun.路径优化(图标)
                if efun.文件是否存在(图标):
                    image = wx.Image(图标).Scale(图标宽度, 图标高度).ConvertToBitmap()
                    self.AddTool(id, 名称, image, shortHelp=帮助文本)
                else:
                    print("工具条图标文件不存在无法创建[{}]文件路径[{}]".format(名称, 图标, ))



"""

.. Hint::
    算数运算

.. literalinclude:: ../../../pyefun/arithmeticOperationBase_test.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-100

"""

import math
import random


def 四舍五入(欲被四舍五入的数值: float, 被舍入的位置: int):
    """
    调用格式： 〈双精度小数型〉 四舍五入 （双精度小数型 欲被四舍五入的数值，［整数型 被舍入的位置］） - 系统核心支持库->算术运算

    英文名称：round

    返回按照指定的方式进行四舍五入运算的结果数值。本命令为初级命令。

    参数<1>的名称为“欲被四舍五入的数值”，类型为“双精度小数型（double）”。

    参数<2>的名称为“被舍入的位置”，类型为“整数型（int）”，可以被省略。如果大于0，表示小数点右边应保留的位数；如果等于0，表示舍入到整数；如果小于0，表示小数点左边舍入到的位置。例如：四舍五入 (1056.65, 1) 返回 1056.7； 四舍五入 (1056.65, 0) 返回 1057； 四舍五入 (1056.65, -1) 返回 1060。如果省略本参数，则默认为0。

    操作系统需求： Windows、Linux

    """
    return round(欲被四舍五入的数值, 被舍入的位置)


def 取绝对值(双精度小数型: float):
    """
     调用格式： 〈双精度小数型〉 取绝对值 （双精度小数型 欲取其绝对值的数值） - 系统核心支持库->算术运算

     英文名称：abs

     如果所提供数值为字节型，则将直接返回该数值。本命令为初级命令。

     参数<1>的名称为“欲取其绝对值的数值”，类型为“双精度小数型（double）”。

     操作系统需求： Windows、Linux
    """
    return abs(双精度小数型)


def 取整(value):
    """
    调用格式： 〈整数型〉 取整 （双精度小数型 欲取整的小数） - 系统核心支持库->算术运算

    英文名称：int

    返回一个小数的整数部分。本命令与“绝对取整”命令不相同之处为：

    如果给定小数为负数，则本命令返回小于或等于该小数的第一个负整数，而“绝对取整”命令则会返回大于或等于该小数的第一个负整数。例如，本命令将 -7.8 转换成 -8，而“绝对取整”命令将 -7.8 转换成 -7。本命令为初级命令。

    参数<1>的名称为“欲取整的小数”，类型为“双精度小数型（double）”。

    操作系统需求： Windows、Linux
    """
    return int(value)


def 求次方(欲求次方数值: float, 次方数: float):
    """
     调用格式： 〈双精度小数型〉 求次方 （双精度小数型 欲求次方数值，双精度小数型 次方数） - 系统核心支持库->算术运算

     英文名称：pow

     返回指定数值的指定次方。本命令为初级命令。

     参数<1>的名称为“欲求次方数值”，类型为“双精度小数型（double）”。参数值指定欲求其某次方的数值。

     参数<2>的名称为“次方数”，类型为“双精度小数型（double）”。参数值指定对欲求次方数值的运算指数。

     操作系统需求： Windows、Linux
    """
    return pow(欲求次方数值, 次方数)


def 求平方根(欲求次方数值: float):
    """
     调用格式： 〈双精度小数型〉 求平方根 （双精度小数型 欲求其平方根的数值） - 系统核心支持库->算术运算

     英文名称：sqr

     返回指定参数的平方根。本命令为初级命令。

     参数<1>的名称为“欲求其平方根的数值”，类型为“双精度小数型（double）”。参数值如果小于零将导致计算错误。

     操作系统需求： Windows、Linux
    """
    return math.sqrt(欲求次方数值)


def 求正弦(欲进行计算的角: float):
    """
     调用格式： 〈双精度小数型〉 求正弦 （双精度小数型 欲进行计算的角） - 系统核心支持库->算术运算

     英文名称：sin

     返回指定角的正弦值。本命令为初级命令。

     参数<1>的名称为“欲进行计算的角”，类型为“双精度小数型（double）”。所使用单位为弧度。为了将角度转换成弧度，请将角度乘以 pi / 180。为了将弧度转换成角度，请将弧度乘以 180 / pi。如果参数值大于等于 2 的 63 次方，或者小于等于 -2 的 63 次方，将导致计算溢出。

     操作系统需求： Windows、Linux
    """
    return math.sin(欲进行计算的角)


def 求余弦(欲进行计算的角: float):
    """
     调用格式： 〈双精度小数型〉 求余弦 （双精度小数型 欲进行计算的角） - 系统核心支持库->算术运算

     英文名称：cos

     返回指定角的余弦值。本命令为初级命令。

     参数<1>的名称为“欲进行计算的角”，类型为“双精度小数型（double）”。所使用单位为弧度。为了将角度转换成弧度，请将角度乘以 pi / 180。为了将弧度转换成角度，请将弧度乘以 180 / pi。如果参数值大于等于 2 的 63 次方，或者小于等于 -2 的 63 次方，将导致计算溢出。

     操作系统需求： Windows、Linux
    """
    return math.cos(欲进行计算的角)


def 求正切(欲进行计算的角: float):
    """
     调用格式： 〈双精度小数型〉 求正切 （双精度小数型 欲进行计算的角） - 系统核心支持库->算术运算

     英文名称：tan

     返回指定角的正切值。本命令为初级命令。

     参数<1>的名称为“欲进行计算的角”，类型为“双精度小数型（double）”。所使用单位为弧度。为了将角度转换成弧度，请将角度乘以 pi / 180。为了将弧度转换成角度，请将弧度乘以 180 / pi。如果参数值大于等于 2 的 63 次方，或者小于等于 -2 的 63 次方，将导致计算溢出。

     操作系统需求： Windows、Linux
    """
    return math.tan(欲进行计算的角)


def 求反正切(欲求其反正切值的数值: float):
    """
     调用格式： 〈双精度小数型〉 求反正切 （双精度小数型 欲求其反正切值的数值） - 系统核心支持库->算术运算

     英文名称：atn

     返回指定数的反正切值。本命令为初级命令。

     参数<1>的名称为“欲求其反正切值的数值”，类型为“双精度小数型（double）”。

     操作系统需求： Windows、Linux
    """
    return math.atan(欲求其反正切值的数值)


def 置随机数种子(欲置入的种子数值):
    """
     调用格式： 〈无返回值〉 置随机数种子 （［整数型 欲置入的种子数值］） - 系统核心支持库->算术运算

     英文名称：randomize

     为随机数生成器初始化一个种子值，不同的种子值将导致“取随机数”命令返回不同的随机数系列。本命令为初级命令。

     参数<1>的名称为“欲置入的种子数值”，类型为“整数型（int）”，可以被省略。如果省略本参数，将默认使用当前计算机系统的时钟值。

     操作系统需求： Windows、Linux
    """
    random.seed()


def 取随机数(欲取随机数的最小值: int, 欲取随机数的最大值: int):
    """
     调用格式： 〈整数型〉 取随机数 （［整数型 欲取随机数的最小值］，［整数型 欲取随机数的最大值］） - 系统核心支持库->算术运算

     英文名称：rnd

     返回一个指定范围内的随机数值。在使用本命令取一系列的随机数之前，应该先使用“置随机数种子”命令为随机数生成器初始化一个种子值。本命令为初级命令。

     参数<1>的名称为“欲取随机数的最小值”，类型为“整数型（int）”，可以被省略。参数必须大于或等于零。本参数如果被省略，默认为 0 。

     参数<2>的名称为“欲取随机数的最大值”，类型为“整数型（int）”，可以被省略。参数必须大于或等于零。本参数如果被省略，默认为无限。

     操作系统需求： Windows、Linux
    """
    return random.randint(欲取随机数的最小值, 欲取随机数的最大值)


def 保留位数(数值, 位数=2):
    """
    保留小数点后指定位数

    :param 数值: 1.12345
    :param 位数: 2
    :return: 1.12
    """
    return format(数值, '.{}f'.format(位数))


def 取最小数(数值列表):
    """
    传入要对比的列表,如(1,2,3),返回里面最小的数字

    :param 数值列表: (1,2,3)
    :return: 1
    """
    return min(数值列表)


def 取最大数(数值列表):
    """
    传入要对比的列表,如(1,2,3),返回里面最大的数字

    :param 数值列表: (1,2,3)
    :return: 3
    """

    return max(数值列表)


def 向下取整(数值):
    """
    示例:1.9返回1

    :param 数值: 1.9
    :return: 1
    """
    return math.floor(数值)


def 向上取整(数值):
    """
    示例:1.1返回2

    :param 数值:  1.1
    :return: 2
    """
    return math.ceil(数值)


def 取整数(val):
    """
    示例:'8852791.5'返回8852791

    :param 字符串:  8852791.5
    :return: 8852791
    """
    return int(float(val))


import unittest

from .ebase64 import *


class TestBase64(unittest.TestCase):

    def test_1(self):
        data = base64编码("123456798")
        self.assertEqual(data, "MTIzNDU2Nzk4")
        data = base64解码(data)
        self.assertEqual(data, b"123456798")

        data = base64编码(b"123456798")
        self.assertEqual(data, "MTIzNDU2Nzk4")
        data = base64解码(data)
        self.assertEqual(data, b"123456798")



"""

.. Hint::
    openpyxl 封装的excel操作

    需要安装 openpyxl

    pip install openpyxl


.. literalinclude:: ../../../example/excel_openpyxl_使用例子.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

from openpyxl import Workbook
from openpyxl import load_workbook
from openpyxl.styles import Font, Alignment  # 导入字体模块
from openpyxl.styles import PatternFill  # 导入填充模块
from openpyxl.styles import Border, Side
from openpyxl.drawing.image import Image



class Excel():
    def 打开Excel(self, 文件路径):
        pass
        self.wb = load_workbook(filename=文件路径)
        self.st = self.wb.active

    def 创建空白工作簿(self):
        pass
        self.wb = Workbook()
        self.st = self.wb.active

    def 置当前sheet名称(self, 名称):
        self.st.title = 名称

    def 取当前sheet对象(self):
        return self.st

    def 取当前sheet名称(self):
        return self.st.title

    def 创建Sheet(self, 名称):
        pass
        self.st = self.wb.create_sheet(名称)

    def 取所有sheet名称(self):
        return self.wb.sheetnames

    def 保存(self, 文件路径):
        self.wb.save(文件路径)

    def 置当前sheet(self, 名称):
        pass
        self.st = self.wb[名称]

    def 取某列的所有内容(self, 第几列):
        rows = self.st.max_row
        column_data = []
        for i in range(1, rows + 1):
            cell_value = self.st.cell(row=i, column=第几列).value
            column_data.append(cell_value)
        return column_data

    def 取某行的所有内容(self, 第几行):
        columns = self.st.max_column
        row_data = []
        for i in range(1, columns + 1):
            cell_value = self.st.cell(row=第几行, column=i).value
            row_data.append(cell_value)
        return row_data

    def 置图片(self, 单元格位置="A1", 宽=100, 高=100, 图片路径=""):
        img = Image(图片路径)
        img.width = 宽
        img.height = 高
        self.st.add_image(img, 单元格位置)

    def 置内容(self, 行, 列, 内容):
        pass
        self.st.cell(row=行, column=列, value=内容)

    def 取内容(self, 行, 列):
        pass
        return self.st.cell(row=行, column=列).value

    def 取行数(self):
        pass
        return self.st.max_row

    def 取列数(self):
        pass
        return self.st.max_column

    def 置列宽(self, 列, 宽度):
        "列 A B C D E "
        self.st.column_dimensions[列].width = 宽度

    def 置行高(self, 行, 高度):
        "行 1 2 3 4 5 "
        self.st.row_dimensions[行].height = 高度

    def 合并单元格(self, 范围="A1:A10"):
        "A2:A10 合并A2到A10之间的单元格"
        self.st.merge_cells(范围)

    def 置单元格背景颜色(self, 行: int, 列: int, 颜色="ffffff"):
        '# 1开始 注意：fill_type为填充类型，如果不写的话，则没有效果。 默认纯色填充 颜色代号 白=ffffff 红=ff0000 黄=ffff00 绿=33ff00 蓝=0000ff 黑=000000 紫=9900ff'
        fill = PatternFill(fill_type='solid', fgColor=颜色)
        self.st.cell(row=行, column=列).fill = fill

    def 置单元格对齐方式(self, 行: int, 列: int, 对齐方式=Alignment(horizontal="justify", vertical="justify")):
        """
        horizontal代表水平方向，可以左对齐left，还有居中center和右对齐right，分散对齐distributed，跨列居中centerContinuous，两端对齐justify，填充fill，常规general

        vertical代表垂直方向，可以居中center，还可以靠上top，靠下bottom，两端对齐justify，分散对齐distributed

        另外还有自动换行：wrap_text，这是个布尔类型的参数，这个参数还可以写作wrapText


        """
        self.st.cell(row=行, column=列).alignment = 对齐方式

    def 置单元格字体风格(self, 行: int, 列: int,
                 字体名称=u"微软雅黑",
                 字体大小=11,
                 加粗=False,
                 斜体=False,
                 删除线=False,
                 颜色="ffffff"):
        ' 白=ffffff 红=ff0000 黄=ffff00 绿=33ff00 蓝=0000ff 黑=000000 紫=9900ff'
        font = Font(字体名称, size=字体大小, bold=加粗, italic=斜体, strike=删除线, color=颜色)
        self.st.cell(row=行, column=列).font = font

    def 置单元格边框(self, 行: int, 列: int,
               left=Side(border_style="thin", color="000000"),
               right=Side(border_style="thin", color="000000"),
               top=Side(border_style="thin", color="000000"),
               bottom=Side(border_style="thin", color="000000"),
               ):
        # dashDot,
        # dashDotDot,
        # dashed,
        # dotted,
        # double,
        # hair,
        # medium,
        # mediumDashDot,
        # mediumDashDotDot,
        # mediumDashed,
        # slantDashDot,
        # thick,
        # thin,
        border = Border(left=left,
                        right=right,
                        top=top,
                        bottom=bottom)
        self.st.cell(row=行, column=列).border = border

    def 删除当前sheet(self, sheet名称=None):
        '不设置名称则删除当前sheet 若设置则删除指定的sheet'
        try:
            if sheet名称 == None:
                self.wb.remove(self.st)
            else:
                del self.wb[sheet名称]
        except IndexError:
            print('最少保留一个表格对象')

    def 关闭(self):
        self.wb.close()




import unittest

from .eqrcode import *


class TestEqrcode(unittest.TestCase):

    def test_1(self):
        pass
        二维码字节集 = 二维码生成("你好 祖国")

        识别结果 = 二维码识别(二维码字节集)

        self.assertEqual(识别结果, "你好 祖国")



import unittest

from .__init__ import *


class TestTypeConv(unittest.TestCase):

    def test_1(self):
        pass
        data = 到文本(b"123")
        print(type(data), data)

        data = 到字节集("123")
        print(type(data), data)

        data = 到整数("123.123")
        print(type(data), data)

        data = 到数值("123.123")
        print(type(data), data)

        data = 到时间("2021-05-18")
        print(type(data), data)

    def test_2(self):
        data = json到文本({"a": 1, "b": 2,"name":"你好"})
        print(data)
        data = json解析(data)
        print(repr(data))
        print(data['a'])
        print(data['b'])
        print(data['name'])



"""

.. Hint::
    时间工具类 时间统计


.. literalinclude:: ../../../pyefun/timeUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

import datetime
import time


class 时间统计():
    """
    计时器，对于需要计时的代码进行with操作：
    with 计时器() as timer:
        ...
        ...
    print(timer.cost)
    ...
    """

    def __init__(self, 名称=""):
        self.开始()
        self.名称 = 名称
        self.zstart = self.start
        if self.名称:
            print("时间统计: %s 开始 %s" % (self.名称, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")))

    def __enter__(self):
        return self

    def 开始(self):
        """
        重新计算开始时间
        """
        self.start = time.perf_counter()

    def 取耗时(self, 名称=""):
        """
        每次计时后重置 如果需要总耗时的话 取总耗时() 即可
        """
        self.end = time.perf_counter()
        self.ms = int((self.end - self.start) * 1000)
        self.开始()
        if 名称 != "":
            print("时间统计: %s %s %sms" % (self.名称, 名称, self.ms))
        elif self.名称 != "":
            print("时间统计: %s %sms" % (self.名称, self.ms))

        return self.ms

    def 取总耗时(self):
        """
        自对象创建以来的总耗时
        """
        self.end = time.perf_counter()
        self.ms = int((self.end - self.zstart) * 1000)
        return self.ms

    def 取毫秒(self):
        return self.取耗时()

    def 取秒(self):
        return self.取耗时() / 1000

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.名称 != "":
            print("时间统计: %s 结束 %sms %s" % (self.名称, self.取总耗时(), datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")))
        return exc_type is None


def 计时器():
    return 时间统计()




import unittest

from .timeUtil import *
from .timeBase import *
from .systemProcessingBase import *


class TestTimeUtil(unittest.TestCase):

    def test_1(self):
        # with 计时器() as t:
        #     延时(1)
        # print(t.取耗时())

        t = 时间统计()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        t.开始()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())

        t = 计时器()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        t.开始()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())



    def test_4(self):
        with 时间统计("买东西业务") as t:
            延时(1)
            t.取耗时("出门")
            延时(1)
            t.取耗时("到店")
            延时(1)
            t.取耗时("购买")
        print("总耗时", t.取总耗时())

    def test_5(self):
        with 时间统计("测试耗时"):
            延时(1)



"""
发送邮件

https://github.com/zhangyunhao116/zmail

pip install zmail
"""
import zmail

def 发送邮件(邮箱,密码,收件邮箱,标题,内容,是否HTML=False):
    '成功返回True 失败返回False'
    if 是否HTML:
        mail = {'subject': 标题,'content_html': 内容}
    else:
        mail = {'subject': 标题,'content_text': 内容}
    try:
        server = zmail.server(邮箱, 密码)
        server.send_mail(收件邮箱, mail, timeout=15)
        return True
    except Exception as e:
        print(e)
        return False


"""
标签组件

"""
import wx
from .wxControl import *
import pyefun.wxefun.evt as evt


class 标签(wx.StaticText, 公用方法):
    pass

    @property
    def 标题(self):
        return self.GetLabel()

    @标题.setter
    def 标题(self, value: str):
        return self.SetLabel(value)

    @property
    def 文字对齐方式(self):
        return ""

    @文字对齐方式.setter
    def 文字对齐方式(self, 标签样式: int):
        """
            wx.标签样式.文字在左边
            wx.标签样式.文字在右边
            wx.标签样式.文字在居中
        """
        self.SetWindowStyle(标签样式)
        self.Refresh()

    @property
    def 省略号位置(self):
        return ""

    @省略号位置.setter
    def 省略号位置(self, 标签样式: int):
        """
            wx.标签样式.省略号在开头
            wx.标签样式.省略号在中间
            wx.标签样式.省略号在末尾
        """
        self.SetWindowStyle(标签样式)
        self.Refresh()

    @property
    def 自动调整大小(self):
        return ""

    @自动调整大小.setter
    def 自动调整大小(self, 是否自动调整大小: bool):
        "默认是自动跳转大小的 如果设置 真 就是默认自动跳转大小 如果设置假就是禁用自动跳转代销"
        if 是否自动调整大小:
            self.ToggleWindowStyle(evt.标签样式.禁用自动调整大小)
        else:
            self.SetWindowStyle(evt.标签样式.禁用自动调整大小)
        self.Refresh()



from .AnimationCtrl import *
from .BitmapButton import *
from .Button import *
from .CalendarCtrl import *
from .CheckBox import *
from .CheckListBox import *
from .ColourPickerCtrl import *
from .ComboBox import *
from .CommandLinkButton import *
from .DatePickerCtrl import *
from .EditableListBox import *
from .FloatSpin import *
from .Frame import *
from .Gauge import *
from .GradientButton import *
from .HyperlinkCtrl import *
from .ListBox import *
from .ListCtrl import *
from .RadioBox import *
from .RadioButton import *
from .ScrollBar import *
from .Slider import *
from .SpinCtrl import *
from .StaticBitmap import *
from .StaticText import *
from .TextCtrl import *
from .ThemedGenBitmapTextButton import *
from .TimePickerCtrl import *
from .Panel import *
from .Timer import *
from .Image import *
from .Sound import *
from .Propgrid import *
from .Notebook import *
from .Toolbar import *
from .Menbar import *



import wx.adv
import wx
from .wxControl import *

class 日历框(wx.adv.CalendarCtrl, 公用方法):
    pass
    @组件_异常检测
    def 突显周末(self,突显=True):
        return self.EnableHolidayDisplay(突显)

    @组件_异常检测
    def 允许修改月份(self,允许=True):
        return self.EnableMonthChange(允许)

    def 取当前日期(self):
        '返回格式：2020/9/26 0:00:00'
        return self.GetDate()

    def 取可选日期范围(self):
        return self.GetDateRange()

    def 取标题背景色(self):
        return self.GetHeaderColourBg()

    def 取标题前景色(self):
        return self.GetHeaderColourFg()

    def 取背景高光颜色(self):
        return self.GetHighlightColourBg()

    def 取前景高光颜色(self):
        return self.GetHighlightColourFg()

    def 取周末突显背景色(self):
        return self.GetHolidayColourBg()

    def 取周末突显前景色(self):
        return self.GetHolidayColourFg()

    @组件_异常检测
    def 标记日期(self,日期,标记=True):
        '日期：0-31'
        self.Mark(日期,标记)

    @组件_异常检测
    def 清除指定日期属性(self,日期):
        '清除与给定日期相关联的所有属性（范围为1…31）。'
        self.ResetAttr(日期)

    @组件_异常检测
    def 置日期属性(self,日期,属性):
        '将属性与指定的日期关联（范围为1…31）'
        return self.SetAttr(日期,属性)

    @组件_异常检测
    def 置当前日期(self,日期):
        '设置当前日期。日期（wx.DateTime）'
        return self.SetDate(日期)

    @组件_异常检测
    def 置可选日期范围(self,最早日期,最晚日期):
        '将可以在控件中选择的日期限制为指定的范围。如果设置了任一日期，则将强制执行并True返回相应的限制。如果未设置，则现有限制将被删除并False返回。日期（wx.DateTime）'
        return self.SetDateRange(最早日期,最晚日期)

    @组件_异常检测
    def 置顶部颜色(self,前景色,背景色):
        '在控件顶部设置用于平日绘画的颜色。该方法当前仅在通用 wx.adv.CalendarCtrl中实现， 在本机版本中不执行任何操作。'
        return self.SetHeaderColours(前景色,背景色)

    @组件_异常检测
    def 置选中日期颜色(self,前景色,背景色):
        '设置用于突出显示当前所选日期的颜色。该方法当前仅在通用 wx.adv.CalendarCtrl中实现， 在本机版本中不执行任何操作。'
        return self.SetHighlightColours(前景色,背景色)

    @组件_异常检测
    def 某天标记为假日(self,日期):
        '将指定的日期标记为当前月份的假日。此方法仅在控件的通用版本中实现，而在本机控件中不执行任何操作。'
        return self.SetHoliday(日期)

    @组件_异常检测
    def 置假日突显颜色(self,前景色,背景色):
        '设置用于假日突出显示的颜色。此方法仅在控件的通用版本中实现，而在本机控件中不执行任何操作。仅当窗口样式包含 CAL_SHOW_HOLIDAYS flag或 EnableHolidayDisplay 已被调用时，才应调用它。'
        return self.SetHolidayColours(前景色,背景色)



import unittest

import pyefun
from pyefun.seleniumUtil import *
import oss as 阿里云oss


class TestOSS(unittest.TestCase):

    def test_1(self):
        环境变量_从文本中加载至系统(读入文本(取运行目录() + "/.env"))

        access_key_id = 取环境变量("access_key_id")
        access_key_secret = 取环境变量("access_key_secret")

        # print(access_key_id, access_key_secret)

        auth = 阿里云oss.获取授权(access_key_id, access_key_secret)
        bucket = 阿里云oss.初始化Bucket(auth, 'oss-cn-hongkong.aliyuncs.com', 'pythonutil')

        浏览器 = 浏览器类()
        # 浏览器.打开chrome()
        浏览器.打开chrome(无头模式=True)
        浏览器.置浏览器位置(左边=0, 顶边=0, 宽度=1366, 高度=768)

        浏览器.浏览网页("https://www.baidu.com")
        print(浏览器.取页面标题())

        images = 浏览器.截图png()
        浏览器.退出()

        bucket.上传文件("test.png", images)

        data = bucket.列举文件("selenium")
        print(data)



"""

.. Hint::
    asyncio 实现的协程池 asyncio为python原生支持

    调用时需要引入

    import asyncio

    还有另外一个版本的实现 asyncPoolGevent 为 Gevent实现协程

.. literalinclude:: ../../../pyefun/asyncPool/asyncPool_test.py
    :language: python
    :caption: 代码示例
    :linenos:

"""

import asyncio
import queue
from concurrent.futures import ThreadPoolExecutor


class 协程池(object):
    """
    1. 支持动态添加任务
    2. 支持停止事件循环
    3. 支持最大协程数：maxsize
    4. 支持进度条
    5. 实时获取剩余协程数
    6. 支持阻塞协程，需要线程进行支持，注意设置线程池：pool_maxsize
    """

    def __init__(self, 协程数量=1, 线程池数量=None, loop=None):
        """
        初始化
        :param loop: 事件循环对象
        :param maxsize: 默认为1
        :param pool_maxsize: 默认为系统内核数
        """
        # 在jupyter需要这个，不然asyncio运行出错
        # import nest_asyncio
        # nest_asyncio.apply()

        # 任务计数器
        self._task, self._task_done_count, self._task_progress_list = self.task_create()

        # 协程池
        self._loop, self._loop_thread, self._thread_pool = self.start_loop(
            loop,
            pool_maxsize=线程池数量)

        # 限制并发量为500
        self.semaphore = asyncio.Semaphore(协程数量, loop=self._loop)

    @staticmethod
    def task_create():
        """
        创建任务对象

        :param
        :return:
        """
        # 队列，先进先出，根据队列是否为空判断，退出协程
        # self.task_done_count: 任务完成后的计数器
        # 任务组，用来阻塞任务用的
        task = queue.Queue()

        # 任务完成后的计数器
        task_done_count = queue.Queue()

        # 任务进度值存储
        # 确保任务值是唯一的
        task_progress_list = []

        return task, task_done_count, task_progress_list

    def task_add(self, item=1):
        """
        添加任务
        :param item:
        :return:
        """
        self._task.put(item)

    def task_done(self, fn):
        """
        任务完成
        回调函数
        已完成任务计数

        :param fn:
        :return:
        """
        if fn:
            pass
        self._task.get()
        self._task.task_done()

        # 已完成任务计数
        self._task_done_count.put(1)

    def task_progress(self, total):
        """
        任务进度条
        适用于已知任务总数的情况

        :param total: 任务总数
        :return:
        """
        # 任务完成计数
        # 需要剔除任务停止的任务数
        count = self._task_done_count.qsize() - 1
        if count < 0:
            count = 0

        item = int(count / total * 100)
        if count == total:
            # 任务完成
            self._task_progress_list.append(100)
        elif item not in self._task_progress_list:
            # 过程值
            self._task_progress_list.append(item)
        else:
            pass

        self._task_progress_list = list(set(self._task_progress_list))
        self._task_progress_list.sort()
        return self._task_progress_list[-1]

    def get_task(self):
        """
        获取事件循环任务列表
        :return:
        """
        task_list = asyncio.Task.all_tasks(loop=self._loop)
        # task_list = asyncio.all_tasks(loop=self._loop)
        return task_list

    def 等待(self):
        """
        任务阻塞
        等待所有任务执行完毕

        :return:
        """
        self._task.join()
        # self._thread_pool.shutdown()

    @property
    def running(self):
        """
        获取剩余协程数
        :return:
        """
        # 剩余任务数
        # 需要剔除任务停止的任务数
        count = self._task.qsize() - 1
        if count < 0:
            count = 0
        return count

    @staticmethod
    def _start_thread_loop(loop):
        """
        运行事件循环
        :param loop: loop以参数的形式传递进来运行
        :return:
        """
        # 将当前上下文的事件循环设置为循环。
        asyncio.set_event_loop(loop)
        # 开始事件循环
        loop.run_forever()

    async def _stop_thread_loop(self, loop_time=1):
        """
        停止协程
        关闭线程
        :return:
        """
        while True:
            if self._task.empty():
                # 停止协程
                self._loop.stop()
                break
            await asyncio.sleep(loop_time)

    def start_loop(self, loop, pool_maxsize=None):
        """
        运行事件循环
        开启新线程
        :param loop: 协程
        :param pool_maxsize: 线程池大小，默认为系统内核数
        :return:
        """
        # 获取一个事件循环
        if not loop:
            loop = asyncio.new_event_loop()

        # 线程池
        thread_pool = ThreadPoolExecutor(pool_maxsize)

        # 设置线程池
        loop.set_default_executor(thread_pool)

        # from threading import Thread
        # thread_pool = Thread(target=self._start_thread_loop, args=(loop,))
        # 设置守护进程
        # thread_pool.setDaemon(True)
        # 运行线程，同时协程事件循环也会运行
        # thread_pool.start()

        # 启动子线程
        # 协程开始启动
        thread_pool.submit(self._start_thread_loop, loop)

        return loop, thread_pool, thread_pool

    def stop_loop(self, loop_time=1):
        """
        队列为空，则关闭线程
        :param loop_time:
        :return:
        """
        # 关闭线程任务
        asyncio.run_coroutine_threadsafe(self._stop_thread_loop(loop_time), self._loop)

        # 取消单个任务
        # task.cancel()

    def _close(self):
        """
        关闭事件循环，不然会重启
        会导致错误
        # RuntimeError: Cannot close a running event loop
        :return:
        """
        self._loop.close()

    def 释放线程(self, loop_time=1):
        """
        释放线程
        :param loop_time:
        :return:
        """
        self.stop_loop(loop_time)

    async def async_semaphore_func(self, func):
        """
        信号代理
        :param func:
        :return:
        """
        async with self.semaphore:
            return await func

    async def async_thread_pool_func(self, block_func, *args, thread_pool=True):
        """
        信号代理
        线程池代理

        loop: asyncio.AbstractEventLoop

        :param block_func: 阻塞任务
        :param args: 参数
        :param thread_pool: 是否使用自定义线程池
        :return:
        """
        async with self.semaphore:
            if not thread_pool:
                # 默认线程池
                return await self._loop.run_in_executor(None, block_func, *args)

            # 使用自定义线程池
            future = await self._loop.run_in_executor(self._thread_pool, block_func, *args)
            # gather = await asyncio.gather(future)  # 阻塞
            # gather = await asyncio.wait(future)  # 阻塞
            return future

    def _submit(self, func, callback=None):
        """
        非阻塞模式
        提交任务到事件循环

        :param func: 异步函数对象
        :param callback: 回调函数
        :return:
        """
        self.task_add()

        # 将协程注册一个到运行在线程中的循环，thread_loop 会获得一个环任务
        # 注意：run_coroutine_threadsafe 这个方法只能用在运行在线程中的循环事件使用
        # future = asyncio.run_coroutine_threadsafe(func, self.loop)
        future = asyncio.run_coroutine_threadsafe(self.async_semaphore_func(func), self._loop)

        # 添加回调函数,添加顺序调用
        if callback:
            future.add_done_callback(callback)
        future.add_done_callback(self.task_done)

    def 投递任务(self, func, *args, 回调函数=None):
        """
        非阻塞模式
        提交任务到事件循环

        :param func: 异步函数对象
        :param args: 参数
        :param callback: 回调函数
        :return:
        """
        self.task_add()

        # 将协程注册一个到运行在线程中的循环，thread_loop 会获得一个环任务
        # 注意：run_coroutine_threadsafe 这个方法只能用在运行在线程中的循环事件使用
        # future = asyncio.run_coroutine_threadsafe(func, self.loop)
        future = asyncio.run_coroutine_threadsafe(self.async_semaphore_func(func(*args)), self._loop)

        # 添加回调函数,添加顺序调用
        if 回调函数:
            future.add_done_callback(回调函数)
        future.add_done_callback(self.task_done)

    def 投递任务2(self, func, *args, 回调函数=None):
        """
        阻塞模式
        提交任务到事件循环

        :param func: 异步函数对象
        :param args: 入参
        :param callback: 回调函数
        :return:
        """
        self.task_add()

        # 将协程注册一个到运行在线程中的循环，thread_loop 会获得一个环任务
        # 注意：run_coroutine_threadsafe 这个方法只能用在运行在线程中的循环事件使用

        # future = self._loop.run_in_executor(None, func)
        # future = asyncio.ensure_future(self._loop.run_in_executor(None, func))  # 非阻塞
        future = asyncio.run_coroutine_threadsafe(
            self.async_thread_pool_func(func, *args),
            self._loop)

        # 添加回调函数,添加顺序调用
        if 回调函数:
            future.add_done_callback(回调函数)
        future.add_done_callback(self.task_done)



import wx.adv
import wx
from .wxControl import *

class 引导按钮(wx.adv.CommandLinkButton, 公用方法):
    pass

    def 取主标题(self):
        return self.GetMainLabel()

    def 取描述内容(self):
        return self.GetNote()

    @组件_异常检测
    def 置标题(self,标题,描述):
        '要设置的标签和注释，两者之间用第一个换行符分隔，或者不设置空白注释'
        return self.SetLabel("{}\n{}".format(标题,描述))

    @组件_异常检测
    def 置标题2(self,标题,描述):
        return self.SetMainLabelAndNote(标题,描述)

    @组件_异常检测
    def 置主标题(self,标题):
        return self.SetMainLabel(标题)

    @组件_异常检测
    def 置描述内容(self,描述):
        return self.SetNote(描述)



import unittest

from .timerit import *
from pyefun.systemProcessingBase import *
from pyefun.arithmeticOperationBase import *


class TestTimeUtil(unittest.TestCase):
    def test_2(self):
        for _ in 计时统计(次数=1000, 显示信息=3):
            math.factorial(10000)

    def test_3(self):
        t1 = 计时统计(次数=10, 标签="运行", 取样=20, 显示信息=3, 单位='ms')
        for timer in t1:
            setup_vars = 10000
            with timer:
                延时(0.01 + 取随机数(1, 2) / 10)
                math.factorial(setup_vars)
        print('总耗时 = %r' % (t1.取耗时(),))


import unittest

from .practical import *


class TestPractical(unittest.TestCase):
    def test12(self):
        data = 取主机名()
        print(data)

        # self.assertEqual(data, "commonlyUtil.py")
        data = 转换为IP地址(取主机名())
        # self.assertEqual(data, "commonlyUtil.py")

        print(data)


    def test11(self):
        data = 标准输出("请输入一个数", ":")

        #data = 标准输入()
        print(data)

    def test14(self):
        data = 结束()

    def test14(self):
        data = 取uuid()
        print(data)
        data = 取短id()
        print(data)
    def test10(self):
        data = 取执行文件名()
        self.assertEqual(data, "practical.py")

        data = 读环境变量("path")
        print(data)

        data = 写环境变量("aaa", "bbb")
        self.assertEqual(data, "bbb")

        data = 读环境变量("aaa")
        self.assertEqual(data, "bbb")


import wx.adv
import wx
from .wxControl import *

class 进度条(wx.Gauge, 公用方法):
    pass

    @property
    def 位置(self):
        return self.取当前位置()

    @位置.setter
    def 位置(self, value):
        return self.置当前位置(value)

    @property
    def 最大位置(self):
        return self.取最大位置()

    @最大位置.setter
    def 最大位置(self, value):
        return self.置最大位置(value)

    def 取最大位置(self):
        return self.GetRange()

    def 取当前位置(self):
        return self.GetValue()

    def 是否为垂直进度条(self):
        return self.IsVertical()

    @组件_异常检测
    def 置最大位置(self,位置):
        return self.SetRange(位置)

    @组件_异常检测
    def 置当前位置(self,位置):
        return self.SetValue(位置)

    def 置加载模式(self):
        '使滚动条编程滚动加载状态,调用SetValue停止滚动加载'
        self.Pulse()



"""

.. Hint::
    进程池使用方法

    方便的使用python开启多进程

.. literalinclude:: ..\..\..\example\进程池测试.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-40

"""
import multiprocessing


class 进程队列:
    def __init__(self):
        self.队列对象 = multiprocessing.Queue()

    def 加入数据(self, 要加入的数据):
        self.队列对象.put(要加入的数据)

    def 获取数据(self):
        return self.队列对象.get()


class 进程:
    def __init__(self, 子程序名, 元组参数=(), 字典参数={}, 进程名=None):
        self.进程对象 = multiprocessing.Process(target=子程序名, args=元组参数, kwargs=字典参数, name=进程名)

    def 启动(self):
        self.进程对象.start()

    def 关闭(self):
        self.进程对象.close()

    def 等待进程(self, 超时时间=None):
        """如果可选参数timeout是None，则该方法将阻塞，直到join()调用其方法的进程终止。如果timeout是一个正数，它最多会阻塞超时秒。请注意，None如果方法的进程终止或方法超时，则返回该方法。检查进程exitcode以确定它是否终止。"""
        self.进程对象.join(超时时间)

    def 取进程名(self):
        return self.进程对象.name

    def 是否存在(self):
        """返回逻辑型"""
        return self.进程对象.is_alive()

    def 取pid(self):
        return self.进程对象.pid

    def 终止子进程(self):
        """子进程的退出代码。None如果流程尚未终止，这将是。负值-N表示孩子被信号N终止。"""
        return self.进程对象.exitcode

    def 守护(self, 是否守护进程=True):
        """ 这个必须在 进程启动先 设置,否则无效  进程的守护进程标志，一个布尔值。必须在start()调用之前设置，当进程退出时，它会尝试终止其所有守护进程子进程。"""
        self.进程对象.daemon = 是否守护进程


class 进程池():
    def __init__(self, 进程数):
        self.进程池对象 = multiprocessing.Pool(processes=进程数)

    def 投递任务(self, 子程序, 元组参数=(), 字典参数={}, 回调函数=None, 回调报错=None):
        启动对象 = self.进程池对象.apply_async(func=子程序, args=元组参数, kwds=字典参数, callback=回调函数, error_callback=回调报错)
        return 启动对象

    def 投递任务2(self, 子程序, 迭代列表):
        """这个用的少,一个子程序报错,全部会报错,后面的函数没有补全了"""
        启动对象 = self.进程池对象.map_async(func=子程序, iterable=迭代列表)
        return 启动对象

    def 等待(self):
        self.停止添加子进程()
        self.等待子进程结束()

    def 停止添加子进程(self):
        """ 防止任何更多的任务被提交到池中。 一旦完成所有任务，工作进程将退出。"""
        self.进程池对象.close()

    def 终止所有子进程(self):
        """立即停止工作进程而不完成未完成的工作。当池对象被垃圾收集时，terminate()将立即调用。；"""
        self.进程池对象.terminate()

    def 等待子进程结束(self):
        """ 等待工作进程退出。必须打电话close()或 terminate()使用之前join()。"""
        self.进程池对象.join()

    def 取返回值(self, 启动对象):
        return 启动对象.get()



import psutil
import pyefun as efun
import os
import subprocess
import pyefun.commonlyUtil as eub
import pyefun.encoding.compress.zip as ezip


class 编译器配置类:
    默认编译器路径 = "C:/efun_view_system/bulidLib/mingw64/bin"


编译器配置 = 编译器配置类()


class 日志类:
    回调日志函数 = print

    def 设置日志回调函数(self, fc):
        self.回调日志函数 = fc

    def 输出(self, info):
        self.回调日志函数(info)


日志 = 日志类()

if efun.系统_是否为window系统():
    import ctypes


@efun.异常处理返回类型逻辑型
def 结束进程和子进程(pid):
    日志.输出("kill proc and all it's subprocesses: {}".format(pid))
    try:
        parent_process = psutil.Process(pid)
    except Exception as ex:
        日志.输出("kill proc failed: " + ex.message)
        return
    parent_process.suspend()
    for p in parent_process.children(recursive=True):
        try:
            日志.输出("kill sub: {}:{} ".format(p.name(), p.pid))
            p.terminate()
        except psutil.NoSuchProcess as ex:
            日志.输出("kill sub proc fail, no such process: " + str(ex.message))
        except Exception as e:
            日志.输出("kill sub: {}:{} failed".format(p.name, p.pid))
            日志.输出(e)
    日志.输出("kill parent: " + parent_process.name())
    try:
        parent_process.kill()
    except Exception:
        日志.输出("kill parent proc: {} failed".format(parent_process.name))


def 运行命令(cmd, 输出=False, 环境变量PATH="", timeout=120):
    global pid

    my_env = os.environ.copy()
    if 环境变量PATH == "":
        环境变量PATH = 编译器配置.默认编译器路径
    my_env["PATH"] = 环境变量PATH + ";" + my_env["PATH"]
    proc = subprocess.Popen(
        cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        stdin=subprocess.PIPE,
        env=my_env,
        close_fds=True,
        cwd=efun.路径优化(efun.取运行目录() + "/bulidLib")
    )
    日志.输出(cmd)
    pid = proc.pid
    proc.stdin.close()
    if 输出:
        try:
            while proc.poll() is None:
                line = proc.stdout.readline()
                line = line.strip()
                if line:
                    日志.输出('{}'.format(line.decode("utf-8")))
        except:
            result = ""
        else:
            try:
                proc = proc.communicate(timeout=timeout)
            except:
                pass
                日志.输出("运行成功 {}".format(proc.pid))
                结束进程和子进程(proc.pid)
                return False
            str = ""
            for v in proc:
                if v != None:
                    try:
                        str = str + v.decode('utf-8')
                    except:
                        pass
            return str
    try:
        result = proc.stdout.read().decode('utf-8')
    except:
        result = ""
    proc.stdout.close()
    return result


def 取gcc版本():
    ret = 运行命令("gcc -v")
    # print(ret)
    if efun.判断文本(ret, ["gcc version"]):
        return efun.strCut(ret, "\r\ngcc version $ ")
    else:
        return '没有找到gcc'


def 取nuitka版本():
    ret = 运行命令("python -m nuitka --version")
    # 日志.输出("检查gcc编译器是否存在 {}".format(ret))
    if efun.判断文本(ret, ["Executable"]):
        return efun.strCut(ret, "$\r\n")
    else:
        return '没有找到nuitka'


def 取python执行路径():
    ret = 运行命令('python -c "import sys;print(sys.executable)"')
    # 日志.输出("检查gcc编译器是否存在 {}".format(ret))
    if efun.判断文本(ret, ["py"]):
        return efun.子文本替换(ret, "\r\n", "")
    else:
        return '没有找到python'


def 取python版本():
    ret = 运行命令('python -V')
    return efun.子文本替换(ret, "\r\n", "")


def 取pythonSitePackages目录():
    ret = 运行命令('python -c "from distutils.sysconfig import get_python_lib;print(get_python_lib())"')
    return efun.子文本替换(ret, "\r\n", "")


def 隐藏控制台窗口():
    """
    Hides the console window in GUI mode. Necessary for frozen application, because
    this application support both, command line processing AND GUI mode and theirfor
    cannot be run via pythonw.exe.
    """
    whnd = ctypes.windll.kernel32.GetConsoleWindow()
    if whnd != 0:
        ctypes.windll.user32.ShowWindow(whnd, 0)
        # if you wanted to close the handles...
        # ctypes.windll.kernel32.CloseHandle(whnd)


def 显示控制台窗口():
    """Unhides console window"""
    whnd = ctypes.windll.kernel32.GetConsoleWindow()
    if whnd != 0:
        ctypes.windll.user32.ShowWindow(whnd, 1)


def 运行设计好的文件(文件路径):
    cmd = r"python {filename}".format(
        filename=文件路径,
    )
    日志.输出("运行 {}".format(cmd))
    ret = 运行命令(cmd, True)
    日志.输出("运行完毕 {}".format(cmd))


def 解压7z(压缩包, 文件路径):
    cmd = efun.路径优化(efun.取运行目录() + r'\bulidLib\7-Zip\7z.exe') + r' x "{}" -o{} -aos -r'.format(压缩包, 文件路径)
    # 日志.输出(cmd)
    运行命令(cmd)


pid = None


def 停止运行():
    global pid
    if pid == None:
        return
    结束进程和子进程(pid)
    pid = None


def 编译_pyinstaller(文件路径):
    编译目录 = efun.文件_取目录(文件路径)
    str = r"""@echo off
cmd pyinstaller -F {name} > build.log
pause
    """.format(dir=编译目录, name=文件路径)
    cmdfile = 编译目录 + "\\编译.bat"
    efun.写到文件(cmdfile, str.encode("gbk"))
    ret = 运行命令("start {}".format(cmdfile))
    日志.输出(ret)
    return ret


def 设置系统PATH环境变量(value):
    "命令太长直接修改错误..坑爹命令"
    command = r'setx PATH "%PATH%;{}"'.format(value)
    proc = subprocess.Popen(
        command,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        stdin=subprocess.PIPE,
    )
    ret = proc.stdout.read().decode("gbk")
    日志.输出(ret)


def _定位模块所在路径(modelName):
    # 编译由于python环境的问题是无法获取系统环境下的文件的
    # 只能用于定位 python运行环境下的
    code = """
import {name}
modelfile = {name}.__file__
""".format(name=modelName)
    loc = locals()
    exec(code, globals(), loc)
    return loc['modelfile']


def 定位模块所在路径(modelName):
    """
    获取系统环境下的python模块所在路径
    """
    code = 'python -c "import {name};print({name}.__file__)"'.format(name=modelName)
    ret = 运行命令(code)
    if efun.判断文本(ret, ["No module", "None"]):
        return ""
    return efun.子文本替换(ret, "\r\n", "")


def cmd回显模式(cmd):
    cmdrtPath = efun.文件从列表中选取存在的文件路径([
        efun.路径优化(r"C:/efun_view_system/resources/cmdrt.exe"),
        efun.路径优化(efun.取运行目录() + r"/resources/cmdrt.exe"),
        efun.路径优化(efun.取运行目录() + r"/bulidLib/cmdrt.exe"),
        efun.路径优化(efun.取运行目录() + r"/cmdrt.exe"),
    ])
    if cmdrtPath == "":
        日志.输出("没有找到文件 cmdrt.exe 请配置易函数视窗编程系统安装包否则可能无法正常运行 现在使用兼容模式")
    else:
        cmd = cmdrtPath + r" " + cmd

    日志.输出(cmd)

    proc = subprocess.Popen(
        cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        stdin=subprocess.PIPE,
        close_fds=True,
        cwd=efun.路径优化(efun.取运行目录() + "/bulidLib")
    )
    try:
        proc = proc.communicate(timeout=5)
    except:
        日志.输出("运行超时自动关闭")
        结束进程和子进程(proc.pid)
        return False
    str = ""
    for v in proc:
        if v != None:
            try:
                str = str + v.decode('utf-8')
            except:
                pass
                日志.输出("内容获取失败")
    return str


def 编译_nuitka(文件路径, 编译目录, 编译器目录):
    cmd = r"nuitka --standalone --mingw64 --show-memory --show-progress --nofollow-imports --follow-import-to=need --output-dir={outdir} {filename}".format(
        filename=文件路径,
        outdir=编译目录
    )
    日志.输出("编译 {}".format(cmd))
    ret = 运行命令(cmd, True, 编译器目录)
    日志.输出("编译完成 {}".format(cmd))


上一次依赖 = ""


def 自动处理依赖缺失问题(文件运行路径, 文件运行目录):
    global 上一次依赖
    运行exe返回结果 = cmd回显模式(文件运行路径)
    if (运行exe返回结果 == False):
        日志.输出('编译完成请检查程序是否运行正常 {}'.format(文件运行路径))
        # os.popen("explorer.exe {}".format(文件运行目录))
        return False
    # 从后面取200个字符大概就可以获取到确实的模块名字了
    运行exe返回结果 = 运行exe返回结果[-200:]
    # 日志.输出(运行exe返回结果)

    model = efun.strCut(运行exe返回结果, "No module named '$'")
    日志.输出("缺少模块 {}".format(model))
    if model == "":
        model = efun.strCut(运行exe返回结果, "AttributeError: module '$' has")
        日志.输出("缺少模块 {}".format(model))
    if model == "":
        日志.输出("运行结果 {}".format(运行exe返回结果))
        日志.输出('编译完成请检查程序是否运行正常 {}'.format(文件运行路径))
        # os.popen("explorer.exe {}".format(文件运行目录))
        return False

    copyfile = ""
    file = 定位模块所在路径(model)
    日志.输出("定位模块所在路径 {}".format(file))
    if file == "":
        # 定位模块失败的话 自动从库目录寻找这个文件
        siteDir = 取pythonSitePackages目录()
        ret = list(eub.查找文件或目录("{}*".format(model), siteDir))
        if (len(ret) == 1):
            日志.输出("找到文件 {}".format(ret[0]))
        else:
            日志.输出("找到多个文件 {}".format(ret))
        copyfile = ret[0]
    if file == None and copyfile == "":
        日志.输出("请手动解决错误 \r\n{}".format(运行exe返回结果))
        日志.输出("解决后重新运行编译")
        return False

    if copyfile == "":
        copyfile = file
        if efun.判断文本(copyfile, ["__init__.py"]):
            copyfile = efun.路径优化(efun.子文本替换(copyfile, "__init__.py", ""))

    if (efun.文件_是否为目录(copyfile)):
        新的目录 = 文件运行目录 + "/" + efun.文件_取文件名(copyfile)
        copyfile = efun.路径优化(copyfile + "/")
        新的目录 = efun.路径优化(新的目录 + "/")
        日志.输出("复制文件夹 {} -> {}".format(copyfile, 新的目录))
        efun.复制目录(copyfile, 新的目录)
    else:
        target = 文件运行目录 + "/" + efun.文件_取文件名(copyfile)
        日志.输出("复制文件 {} -> {}".format(copyfile, target))
        efun.复制文件(copyfile, target)
    if (上一次依赖 != copyfile):
        上一次依赖 = copyfile
    else:
        return False
    return True


def 编译文件(文件路径, 编译目录="", 编译器目录="", 不编译=False, 不寻找依赖=False):
    文件路径 = efun.路径优化(文件路径)
    文件目录 = efun.路径优化(efun.文件_取目录(文件路径))
    文件名 = efun.strCut(efun.文件_取文件名(文件路径), "$.")
    if 编译目录 == "":
        编译目录 = efun.路径优化(efun.文件_取目录(文件路径) + r"/o")
    if 编译器目录 == "":
        编译器目录 = 编译器配置.默认编译器路径
    文件运行路径 = efun.路径优化(r"{1}\{0}.dist\{0}.exe".format(文件名, 编译目录))
    文件运行目录 = efun.路径优化(efun.文件_取目录(文件运行路径))
    # 日志.输出("依赖目录 {}".format(依赖目录))
    日志.输出("文件路径 {}".format(文件路径))
    日志.输出("文件名 {}".format(文件名))
    日志.输出("编译目录 {}".format(编译目录))
    日志.输出("文件运行路径 {}".format(文件运行路径))
    日志.输出("文件运行目录 {}".format(文件运行目录))
    if 不编译 == False:
        编译_nuitka(文件路径, 编译目录, 编译器目录)
    if 不寻找依赖 == False:
        日志.输出("复制资源文件夹{} -> {}".format(efun.路径优化(文件目录 + "/resources"), efun.路径优化(文件运行目录 + "/resources")))
        efun.复制目录(efun.路径优化(文件目录 + "/resources"), efun.路径优化(文件运行目录 + "/resources"))
        while 自动处理依赖缺失问题(文件运行路径, 文件运行目录):
            pass


# """
# C:\Users\xxxx\AppData\Local\Nuitka\Nuitka\ccache\v3.7.12
# ccache-3.7.12-windows-32
#
# C:\Users\\AppData\Local\Nuitka\Nuitka\depends\x86_64
# depends22_x64.zip
# https://pypi.org/project/Nuitka/#history
# python -m pip install nuitka
#
# pip install nuitka
# PS C:\pyefun\wxview\create_test> pip uninstall nuitka
# Found existing installation: Nuitka 0.6.15.3
# Uninstalling Nuitka-0.6.15.3:
#   Would remove:
#     c:\users\\anaconda3\lib\site-packages\nuitka-0.6.15.3.dist-info\*
#     c:\users\\anaconda3\lib\site-packages\nuitka\*
#     c:\users\\anaconda3\scripts\nuitka-run.bat
#     c:\users\\anaconda3\scripts\nuitka.bat
#     c:\users\\anaconda3\scripts\nuitka3-run.exe
#     c:\users\\anaconda3\scripts\nuitka3.exe
# """
def 初始化编译环境():
    """该命名需要安装易函数视窗编程系统 否则代码中的文件路径将会找不到"""
    压缩包路径_ccache = efun.路径优化(efun.取运行目录() + "/bulidLib/ccache-3.7.12-windows-32.zip")
    压缩包路径_depends = efun.路径优化(efun.取运行目录() + "/bulidLib/depends22_x64.zip")
    压缩包路径_gcc = efun.路径优化(efun.取运行目录() + "/bulidLib/mingw64.7z")
    filelist = list(eub.查找文件或目录("Nuitka*", efun.取运行目录() + "/bulidLib/Nuitka/"))
    try:
        pip安装包路径 = efun.路径优化(filelist[0])
    except:
        pip安装包路径 = "nuitka"

    安装目录 = eub.系统_设置应用缓存目录("Nuitka", "Nuitka")
    安装目录_ccache = efun.路径优化(安装目录 + "/ccache")
    安装目录_depends = efun.路径优化(安装目录 + "/depends/x86_64")
    安装目录_gcc = efun.路径优化(efun.取运行目录() + "/bulidLib/mingw64")
    安装目录_gcc_2 = efun.路径优化(efun.取运行目录() + "/bulidLib")

    日志.输出("安装目录 {}".format(安装目录))
    日志.输出("安装目录_ccache {}".format(安装目录_ccache))
    日志.输出("安装目录_depends {}".format(安装目录_depends))
    日志.输出("压缩包路径_ccache {}".format(压缩包路径_ccache))
    日志.输出("压缩包路径_depends {}".format(压缩包路径_depends))

    # C:/Users\xxx\AppData\Local\Nuitka\Nuitka\ccache\v3.7.12
    # ccache-3.7.12-windows-32
    #
    # C:\Users\\AppData\Local\Nuitka\Nuitka\depends\x86_64
    # depends22_x64.zip

    # C:\efun_view_system\bulidLib
    # ccache-3.7.12-windows-32.zip
    efun.设置_异常处理_显示信息()
    if not efun.文件是否存在(安装目录_ccache):
        ezip.zip解压(压缩包路径_ccache, 安装目录_ccache)

    if not efun.文件是否存在(安装目录_depends):
        ezip.zip解压(压缩包路径_depends, 安装目录_depends)

    ret = 运行命令("gcc -v")
    日志.输出("检查gcc编译器是否存在 {}".format(ret))
    if efun.判断文本(ret, ["gcc version"]):
        日志.输出("gcc环境通过")
    else:
        日志.输出("gcc 未安装自动安装中")
        if not efun.文件是否存在(安装目录_gcc):
            解压7z(压缩包路径_gcc, 安装目录_gcc_2)

            # ezip.zip解压(压缩包路径_gcc, 安装目录_gcc)

        ret = 运行命令("gcc -v")
        日志.输出("检查gcc编译器是否存在 {}".format(ret))
        if efun.判断文本(ret, ["gcc version"]):
            日志.输出("gcc环境通过")
        else:
            日志.输出("gcc环境配置失败请自行处理后再编译")
            return False

    ret = 运行命令("python -m nuitka --version")
    日志.输出("检查编译器是否存在 {}".format(ret))
    if efun.判断文本(ret, ["No module named nuitka"]):
        日志.输出("安装命令 {}".format("python -m pip install {}".format(pip安装包路径)))

        ret = 运行命令("python -m pip install {}".format(pip安装包路径))
        日志.输出("安装结果 {}".format(ret))

        ret = 运行命令("python -m nuitka --version")
        日志.输出("检查编译器是否存在 {}".format(ret))

    if efun.判断文本(ret, ["Executable"]):
        日志.输出("环境配置成功")
        return True
    else:
        日志.输出("环境配置失败请自行处理")
        return False



"""
播放器组件

https://www.wxpython.org/Phoenix/docs/html/wx.adv.Sound.html

"""

import wx
from .wxControl import *


class 播放器(wx.adv.Sound, 公用方法):
    def __init__(self, 文件路径):
        super(播放器, self).__init__(文件路径)

    def 从音乐文件中加载(self, 文件路径):
        return self.Create(文件路径)

    def 从音乐数据中加载(self, data):
        return self.CreateFromData(data)

    def 是否已加载(self):
        return self.IsOk()

    def 播放音乐(self, 播放次数):
        """
        wx.播放次数.仅播放一次
        wx.播放次数.循环播放

        """
        return self.Play(播放次数)

    def 播放音乐文件(self, 文件路径, 播放次数):
        """
        播放声音文件。

        如果正在播放另一种声音，它将被中断。

        True成功返回，False否则返回。请注意，通常可以在调用此函数后随时删除正在异步播放的对象，声音将继续播放，但是对于从内存数据加载的声音对象，这在 Windows 下目前不起作用。
        """
        return self.PlaySound(文件路径, 播放次数)

    def 停止播放(self,):
        return self.Stop()


"""

.. Hint::
    数组操作

.. literalinclude:: ../../../pyefun/arrayActionBase_test.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-100

"""

class 数组:

    def __init__(self, data: list = []):
        self.val = list(data)

    def 加入成员(self, object):
        self.val.append(object)

    def 统计成员次数(self, object):
        return self.val.count(object)

    def 查找成员(self, object):
        return self.val.index(object)

    def 弹出成员(self, index: int = -1):
        return self.val.pop(index)

    def 插入成员(self, object, index: int = -1):
        self.val.insert(index, object)

    def 移除成员(self, object):
        self.val.remove(object)

    def 翻转(self):
        self.val.reverse()

    def 排序(self, **kwargs):
        # cmp=None, key=None, reverse=False
        # cmp -- 可选参数, 如果指定了该参数会使用该参数的方法进行排序。
        # key -- 主要是用来进行比较的元素，只有一个参数，具体的函数的参数就是取自于可迭代对象中，指定可迭代对象中的一个元素来进行排序。
        # reverse -- 排序规则，reverse = True 降序， reverse = False 升序（默认）。
        self.val.sort(**kwargs)

    def 从大到小(self, 下标=0):
        pass
        self.排序(reverse=True, key=lambda d: _func_key(d, 下标))

    def 从小到大(self, 下标=0):
        self.排序(reverse=False, key=lambda d: _func_key(d, 下标))

    def 取所有成员(self):
        return self.val

    def 清空(self):
        self.val = []


def _func_key(d, 下标=0):
    if (type(d) == tuple):
        return d[下标]
    return d


def 字典_取值并删除(字典, 键, 失败返回值=None):
    '如果查找键不存在则返回设置的失败返回值,该值可空'
    return 字典.pop(键, 失败返回值)


def 字典_取指定键值(字典, 键, 失败返回值=None):
    '如果查找键不存在则返回设置的失败返回值'
    return 字典.get(键, 失败返回值)


def 字典_清空(字典):
    '清空字典内的全部元素,成功返回True'
    字典.clear()
    return True


def 字典_拷贝(新字典, 原字典):
    '成功返回True 直接赋值拷贝值会跟着原字典改变,用copy不会'
    新字典 = 原字典.copy()
    return True


def 字典_生成(键值列表, 键值):
    '传入键值列表创建字典,字典内的值都为设置的键值'
    return dict.fromkeys(键值列表, 键值)


def 字典_转列表(字典):
    '返回列表格式[(1,2),(2,3),(3,4)]'
    return list(字典.items())


def 字典_取全部键(字典):
    return list(字典.keys())


def 字典_取全部值(字典):
    return list(字典.values())


def 字典_取出并删除最后键值(字典):
    '删除字典中最后一个键跟值并以元组格式返回删除的键跟值'
    return 字典.popitem()


def 字典_取值添加(字典, 键, 值=None):
    '如果查找键不存在则返回设置的失值且为字典新建该键值'
    return 字典.setdefault(键, 值)


def 列表_转字典(列表):
    '将[(1,2),(3,4)]转换成{1:2,3:4}'
    字典 = dict()
    for x in 列表: 字典[x[0]] = x[1]
    return 字典


def 列表_合并为字典(列表1, 列表2):
    '传入两个列表转换成字典 [1,2],[8,9]==>{1:8,2:9}'
    return dict(zip(列表1, 列表2))


def 列表_加入成员(列表, 值):
    '成功返回True'
    列表.append(值)
    return True


def 列表_插入成员(列表, 位置, 值):
    '成功返回True 在指定位置插入指定值'
    列表.insert(位置, 值)
    return True


def 列表_取出现次数(列表, 值):
    '搜索时 True 会当成1   False 是0'
    return 列表.count(值)


def 列表_合并列表(列表, 新列表):
    '成功返回True 在列表后面追加新的列表或元组成员进去'
    列表.extend(新列表)
    return True


def 列表_查找成员位置(列表, 值):
    return 列表.index(值)


def 列表_取值并删除(列表, 位置=None):
    '取出列表的一个成员值 并删除该成员,默认最后一个,位置为0则为第一个'
    if 位置 == None:
        return 列表.pop()
    else:
        return 列表.pop(位置)


def 列表_删除指定值(列表, 值):
    '成功返回True 删除列表中找到的第一个值'
    列表.remove(值)
    return True


def 列表_倒序排列(列表):
    '成功返回True 把列表的成员顺序到过来排序'
    列表.reverse()
    return True


def 列表_大小排序(列表, 排序方式=False):
    '成功返回True 排序的列表只能全为整数型的,排序方式True为从大到小,默认False从小到大'
    列表.sort(reverse=排序方式)
    return True


def 数组_按成员长度排序(数组):
    '传入一个序列，根据成员的长度排序 长的在前面'
    return sorted(数组, key=lambda i: len(i), reverse=True)


def 数组_按子成员大小排序(数组, 成员索引):
    '处理数组内包含数组需要排序的'
    return sorted(数组, key=lambda i: i[成员索引])


def 数组_取随机成员数组(数组, 数量):
    '失败返回False,在指定数组内随机取出指定数量的成员组成新数组返回'
    return random.sample(数组, 数量)


def 数组_取随机成员(数组):
    '可以传入字符 元组 列表登录,随机取出一个值'
    return random.choice(数组)



import wx
from .wxControl import *


class 组合框(wx.ComboBox, 公用方法):

    @property
    def 宽度(self):
        size = self.GetSize()
        return size[0]

    @宽度.setter
    def 宽度(self, value):
        size = self.GetSize()
        size[0] = value
        self.SetSize(-1, -1, value, size[1])

    @property
    def 高度(self):
        size = self.GetSize()
        return size[1]

    @高度.setter
    def 高度(self, value):
        size = self.GetSize()
        size[1] = value
        self.SetSize(-1, -1, size[0], value)

    @property
    def 现行选中项(self):
        return self.GetSelection()

    @现行选中项.setter
    def 现行选中项(self, value):
        self.SetSelection(value)

    @property
    def 列表项目(self):
        return []

    @列表项目.setter
    def 列表项目(self, value):
        print(value)
        self.Set(value)

    @property
    def 内容(self):
        return self.GetValue()

    @内容.setter
    def 内容(self, value):
        self.SetValue(value)

    @组件_异常检测
    def 取指定项目索引(self, 项目文本, 是否区分大小写=False):
        return self.FindString(项目文本, 是否区分大小写)

    def 取项目数(self):
        return self.GetCount()

    def 取选中项索引(self):
        return self.GetCurrentSelection()

    def 取选中项索引2(self):
        return self.GetSelection()

    def 取选中范围(self):
        return self.GetTextSelection()

    @组件_异常检测
    def 取项目文本(self, 索引):
        return self.GetString(索引)

    def 取选中项文本(self):
        return self.GetStringSelection()

    def 列表项是否为空(self):
        return self.IsListEmpty()

    def 文本是否为空(self):
        return self.IsTextEmpty()

    def 弹出列表(self):
        self.Popup()

    @组件_异常检测
    def 置项目文本(self, 索引, 文本):
        self.SetString(索引, 文本)

    @组件_异常检测
    def 置默认文本(self, 文本):
        self.SetValue(文本)

    @组件_异常检测
    def 置选中项(self, 索引):
        self.SetSelection(索引)

    @组件_异常检测
    def 置选中项_文本(self, 项目文本):
        return self.SetStringSelection(项目文本)

    @组件_异常检测
    def 选中范围文本(self, 开始位置, 结束位置):
        '如果两个参数都等于-1，则选择控件中的所有文本'
        self.SetTextSelection(开始位置, 结束位置)

    @组件_异常检测
    def 清空(self):
        """
            删除组合框列表部分中的所有项目。
        """
        self.Clear()

    @组件_异常检测
    def 置项目列表(self, 项目列表):
        '会覆盖原有的项目列表'
        self.SetItems(项目列表)

    @组件_异常检测
    def 加入项目(self, 项目文本: str):
        '支持单个或多个项目,多个项目使用列表传入，加入后会返回最后一个项目索引'
        return self.Append(项目文本)

    @组件_异常检测
    def 删除项目(self, 项目索引):
        self.Delete(项目索引)

    @组件_异常检测
    def 插入项目(self, 欲插入的位置, 欲插入项目的文本):
        return self.Insert(欲插入项目的文本, 欲插入的位置)

    @组件_异常检测
    def 置列表项目(self, 列表项目列表):
        """
        用给定的项目替换当前控件内容。
        请注意，如果您需要添加大量项目，调用此方法通常比将它们一个一个地附加起来要快得多。
        """
        self.Clear()
        self.Set(列表项目列表)



import unittest

from .egzip import *
import binascii


class Testegzip(unittest.TestCase):

    def test_1(self):
        data = gzip压缩(b"1234567890")
        print("压缩后大小 %s" % len(data)  )

        ddata = gzip解压(data)

        self.assertEqual(ddata, b"1234567890")

        # print('压缩的数据:长度 : {},内容 : {}'.format(len(data), ddata))



import wx.adv
import wx
from .wxControl import *

class 排序列表框(wx.adv.EditableListBox, 公用方法):
    pass

    def 取项目列表(self):
        return self.GetStrings()

    @组件_异常检测
    def 置项目列表(self,项目列表):
        return self.SetStrings(项目列表)
